# -*- coding: utf-8 -*-
"""TcEx Framework"""
import json
import os
import sys
from argparse import Namespace

from .tcex_argparser import TcExArgParser


class TcExArgs(object):
    """Module for handling args passed to App from CLI, SecureParams, and AOT"""

    def __init__(self, tcex):
        """Initialize Class Properties.

        Args:
            tcex (tcex.TcEx): Instance of TcEx class.
        """

        self.tcex = tcex
        self._config_data = {}
        self._default_args = None
        self._default_args_resolved = None
        self._parsed = False
        self._parsed_resolved = False
        self.parser = TcExArgParser()

        # NOTE: odd issue where args is not updating properly
        # if self.default_args.tc_token is not None:
        #     self._tc_token = self.default_args.tc_token
        # if self.default_args.tc_token_expires is not None:
        #     self._tc_token_expires = self.default_args.tc_token_expires

    def _load_secure_params(self):
        """Load secure params from the API.

        # API Response:

        .. code-block:: javascript
            :linenos:
            :lineno-start: 1

            {
                "inputs":
                    {
                        "tc_playbook_db_type": "Redis",
                        "fail_on_error": true,
                        "api_default_org": "TCI"
                    }
            }

        Returns:
            dict: Parameters ("inputs") from the TC API.
        """
        self.tcex.log.info('Loading secure params.')
        # Retrieve secure params and inject them into sys.argv
        r = self.tcex.session.get('/internal/job/execution/parameters')

        # check for bad status code and response that is not JSON
        if not r.ok or r.headers.get('content-type') != 'application/json':
            err = r.text or r.reason
            self.tcex.exit(1, 'Error retrieving secure params from API ({}).'.format(err))

        # return secure params
        return r.json().get('inputs', {})

    def _results_tc_args(self):
        """Read data from results_tc file from previous run of app.

        This method is only required when not running from the with the
        TcEX platform and is only intended for testing apps locally.

        Returns:
            (dictionary): A dictionary of values written to results_tc.
        """
        results = []
        if os.access(self.default_args.tc_out_path, os.W_OK):
            result_file = '{}/results.tc'.format(self.default_args.tc_out_path)
        else:
            result_file = 'results.tc'
        if os.path.isfile(result_file):
            with open(result_file, 'r') as rh:
                results = rh.read().strip().split('\n')
            os.remove(result_file)
        for line in results:
            if not line or ' = ' not in line:
                continue
            key, value = line.split(' = ')
            if value == 'true':
                value = True
            elif value == 'false':
                value = False
            elif not value:
                value = None
            setattr(self._default_args, key, value)

    def _unknown_args(self, args):
        """Log argparser unknown arguments.

        Args:
            args (list): List of unknown arguments
        """
        for u in args:
            self.tcex.log.warning(u'Unsupported arg found ({}).'.format(u))

    def args(self):
        """Parse args if they have not already been parsed and return the Namespace for args.

        .. Note:: Accessing args should only be done directly in the App.

        Returns:
            (namespace): ArgParser parsed arguments.
        """
        if not self._parsed:  # only resolve once
            self._default_args, unknown = self.parser.parse_known_args()

            # when running locally retrieve any args from the results.tc file.  when running in
            # platform this is done automatically.
            self._results_tc_args()

            # log unknown arguments only once
            self._unknown_args(unknown)

            # set parsed bool to ensure args are only parsed once
            self._parsed = True

            # update args with value from config data or configuration file
            self.args_update()

            # reinitialize logger with new log level and api settings
            self.tcex.logger.remove_handler_by_name('sh')
            self.tcex.logger.add_rotating_file_handler()

            # log system and App data
            self.tcex.logger.log_info()

        return self._default_args

    def args_update(self):
        """Update the argparser namespace with any data from configuration file."""
        for key, value in self._config_data.items():
            setattr(self._default_args, key, value)

    def config(self, config_data):
        """Add configuration data to be injected into sys.argv.

        Below are the default args that the TcEx frameworks supports. Any App specific args
        should be included in the provided data.

        .. code-block:: javascript

            {
              "api_access_id": "$env.API_ACCESS_ID",
              "api_default_org": "$env.API_DEFAULT_ORG",
              "api_secret_key": "$envs.API_SECRET_KEY",
              "tc_api_path": "$env.TC_API_PATH",
              "tc_log_level": "debug",
              "tc_log_path": "log",
              "tc_owner": "MyOwner",
              "tc_proxy_host": "$env.TC_PROXY_HOST",
              "tc_proxy_password": "$envs.TC_PROXY_PASSWORD",
              "tc_proxy_port": "$env.TC_PROXY_PORT",
              "tc_proxy_tc": false,
              "tc_proxy_username": "$env.TC_PROXY_USERNAME"
            }

        Args:
            config (dict): A dictionary of configuration values.
        """
        self._config_data = config_data

    def config_file(self, filename):
        """Load configuration data from provided file and inject values into sys.argv.

        Args:
            config (str): The configuration file name.
        """
        if os.path.isfile(filename):
            with open(filename, 'r') as fh:
                self._config_data = json.load(fh)
        else:
            self.tcex.log.error('Could not load configuration file "{}".'.format(filename))

    @property
    def default_args(self):
        """Parse args and return default args."""
        if self._default_args is None:
            self._default_args, unknown = self.parser.parse_known_args()  # pylint: disable=W0612

            if self._default_args.tc_aot_enabled:
                # block for AOT message and get params
                params = self.tcex.playbook.aot_blpop()
                self.inject_params(params)
            elif self._default_args.tc_secure_params:
                # inject secure params from API
                params = self._load_secure_params()
                self.inject_params(params)
            else:
                # reinitialize logger with new log level and api settings
                self.tcex.logger.add_rotating_file_handler()

        return self._default_args

    def inject_params(self, params):
        """Inject params into sys.argv from secureParams API, AOT, or user provided.

        Args:
            params (dict): A dictionary containing all parameters that need to be injected as args.
        """

        for arg, value in params.items():
            cli_arg = '--{}'.format(arg)
            if cli_arg in sys.argv:
                # arg already passed on the command line
                self.tcex.log.debug('skipping existing arg: {}'.format(cli_arg))
                continue

            # ThreatConnect secure/AOT params should be updated in the future to proper JSON format.
            # MultiChoice data should be represented as JSON array and Boolean values should be a
            # JSON boolean and not a string.
            delimiter = self.tcex.install_json.get('listDelimiter', '|')
            param_data = self.tcex.install_json_params.get(arg) or {}
            if param_data.get('type', '').lower() == 'multichoice' or param_data.get(
                'allowMultiple'
            ) in ['true', True]:
                # update delimited value to an array for params that have type of MultiChoice.
                value = value.split(delimiter)
            elif param_data.get('type', '').lower() == 'boolean':
                # update value to be a boolean instead of string "true"/"false".
                value = self.tcex.utils.to_bool(value)
            elif arg in self.tc_bool_args:
                value = self.tcex.utils.to_bool(value)

            if isinstance(value, (bool)):
                # handle bool values as flags (e.g., --flag) with no value
                if value is True:
                    sys.argv.append(cli_arg)
            elif isinstance(value, (list)):
                for mcv in value:
                    sys.argv.append('{}={}'.format(cli_arg, mcv))
            else:
                sys.argv.append('{}={}'.format(cli_arg, value))

        # reset default_args now that values have been injected into sys.argv
        self._default_args, unknown = self.parser.parse_known_args()  # pylint: disable=W0612

        # reinitialize logger with new log level and api settings
        self.tcex.logger.add_rotating_file_handler()

    def resolved_args(self):
        """Parse args if they have not already been parsed and return the Namespace for args.

        .. Note:: Accessing args should only be done directly in the App.

        Returns:
            (namespace): ArgParser parsed arguments with Playbook variables automatically resolved.
        """

        if not self._parsed_resolved:  # only resolve once
            self.args()

            # create new args Namespace for resolved args
            self._default_args_resolved = Namespace()

            # iterate over args and resolve any playbook variables
            for arg in vars(self._default_args):
                arg_val = getattr(self._default_args, arg)
                if arg not in self.tc_reserved_args:
                    if isinstance(arg_val, (str)):
                        arg_val = self.tcex.playbook.read(arg_val)
                setattr(self._default_args_resolved, arg, arg_val)

            # set parsed bool to ensure args are only parsed once
            self._parsed_resolved = True

        return self._default_args_resolved

    @property
    def tc_bool_args(self):
        """Return a list of default ThreatConnect Args that are booleans."""
        return [
            'apply_proxy_external',
            'apply_proxy_ext',
            'apply_proxy_tc',
            'batch_halt_on_error',
            'tc_aot_enabled',
            'tc_log_to_api',
            'tc_proxy_external',
            'tc_proxy_tc',
            'tc_secure_params',
            'tc_verify',
        ]

    @property
    def tc_reserved_args(self):
        """Return a list of *all* ThreatConnect reserved arg values."""
        return [
            'tc_token',
            'tc_token_expires',
            'api_access_id',
            'api_secret_key',
            'batch_action',
            'batch_chunk',
            'batch_halt_on_error',
            'batch_poll_interval',
            'batch_interval_max',
            'batch_write_type',
            'tc_playbook_db_type',
            'tc_playbook_db_context',
            'tc_playbook_db_path',
            'tc_playbook_db_port',
            'tc_playbook_out_variables',
            'api_default_org',
            'tc_api_path',
            'tc_in_path',
            'tc_log_file',
            'tc_log_path',
            'tc_out_path',
            'tc_secure_params',
            'tc_temp_path',
            'tc_user_id',
            'tc_proxy_host',
            'tc_proxy_port',
            'tc_proxy_username',
            'tc_proxy_password',
            'tc_proxy_external',
            'tc_proxy_tc',
            'tc_log_to_api',
            'tc_log_level',
            'logging',
        ]



# -*- coding: utf-8 -*-
"""Test the TcEx Batch Module."""

import pytest

from ..tcex_init import tcex


# pylint: disable=R0201,W0201
class TestIndicator1:
    """Test the TcEx Batch Module."""

    def setup_class(self):
        """Configure setup before all tests."""

    @pytest.mark.parametrize(
        'indicator,description,label,tag',
        [
            ('1.11.111.1', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('1.11.111.2', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('1.11.111.3', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('1.11.111.4', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_address(self, indicator, description, label, tag):
        """Test address creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'address', indicator])
        ti = batch.address(ip=indicator, rating='5.0', confidence='100', xid=xid)
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'indicator,description,label,tag',
        [
            ('pytest-email_address-i1-001@test.com', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-email_address-i1-002@test.com', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-email_address-i1-003@test.com', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-email_address-i1-004@test.com', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_email_address(self, indicator, description, label, tag):
        """Test email_address creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'email_address', indicator])
        ti = batch.email_address(address=indicator, rating='5.0', confidence='100', xid=xid)
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'md5,sha1,sha256,description,label,tag',
        [
            ('a1', 'a1', 'a1', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('b1', 'b1', 'b1', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('c1', 'c1', 'c1', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('d1', 'd1', 'd1', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_file(self, md5, sha1, sha256, description, label, tag):
        """Test file creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'file', md5, sha1, sha256])
        ti = batch.file(
            md5=md5 * 16,
            sha1=sha1 * 20,
            sha256=sha256 * 32,
            rating='5.0',
            confidence='100',
            xid=xid,
        )
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'indicator,description,label,tag',
        [
            ('pytest-host-i1-001.com', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-host-i1-002.com', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-host-i1-003.com', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-host-i1-004.com', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_host(self, indicator, description, label, tag):
        """Test host creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'host', indicator])
        ti = batch.host(hostname=indicator, rating='5.0', confidence='100', xid=xid)
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'indicator,description,label,tag',
        [
            ('https://pytest-url-i1-001.com', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('https://pytest-url-i1-002.com', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('https://pytest-url-i1-003.com', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('https://pytest-url-i1-004.com', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_url(self, indicator, description, label, tag):
        """Test url creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'url', indicator])
        ti = batch.url(text=indicator, rating='5.0', confidence='100', xid=xid)
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""TcEx Framework Profile Generation Module."""
import json
import os
import re
import sys
from random import randint

try:
    from mako.template import Template
except ImportError:
    pass  # mako is only required for local testing
import requests
import colorama as c

from .tcex_bin import TcExBin

BASE_URL = 'https://raw.githubusercontent.com/ThreatConnect-Inc/tcex'


class Profiles:
    """Profile Class

    Raises:
        NotImplementedError: The delete method is not currently implemented.
        NotImplementedError: The update method is not currently implemented.
    """

    def __init__(self, profile_dir):
        """Initialize class properties."""
        self.profile_dir = profile_dir

        # properties
        self._profiles = None

    def add(self, profile_name, profile_data, sort_keys=True):
        """Add a profile."""
        profile_filename = os.path.join(self.profile_dir, '{}.json'.format(profile_name))
        if os.path.isfile(profile_filename):
            print(
                '{}{}A profile with the name ({}) already exists.'.format(
                    c.Style.BRIGHT, c.Fore.RED, profile_name
                )
            )
            sys.exit(1)

        profile = {
            'exit_codes': profile_data.get('exit_codes', [0]),
            'outputs': profile_data.get('outputs'),
            'stage': profile_data.get('stage', {'redis': {}, 'threatconnect': {}}),
        }
        if profile_data.get('runtime_level').lower() == 'triggerservice':
            profile['configs'] = profile_data.get('configs')
            profile['trigger'] = profile_data.get('trigger')
        else:
            profile['inputs'] = profile_data.get('inputs')

        if profile_data.get('runtime_level').lower() == 'organization':
            profile['validation_criteria'] = profile_data.get('validation_criteria', {'percent': 5})
            profile.pop('outputs')

        with open(profile_filename, 'w') as fh:
            json.dump(profile, fh, indent=2, sort_keys=sort_keys)

    def delete(self, profile_name):
        """Delete an existing profile."""
        raise NotImplementedError('The delete method is not currently implemented.')

    def update(self, profile_name):
        """Update an existing profile."""
        raise NotImplementedError('The update method is not currently implemented.')


class Validation:
    """Validation Class

    Raises:
        NotImplementedError: The delete method is not currently implemented.
        NotImplementedError: The update method is not currently implemented.
    """

    def __init__(self, base_dir, branch='master'):
        """Initialize class properties."""
        self.branch = branch
        self.base_dir = base_dir
        self._variable_match = re.compile(r'^{}$'.format(self._variable_pattern))
        self._variable_parse = re.compile(self._variable_pattern)

    def _method_name(self, variable):
        """Convert variable name to a valid method name.

        Args:
            variable (string): The variable name to convert.

        Returns:
            (str): Method name
        """
        method_name = None
        if variable is not None:
            variable = variable.strip()
            if re.match(self._variable_match, variable):
                var = re.search(self._variable_parse, variable)
                variable_name = var.group(3).replace('.', '_').lower()
                variable_type = var.group(4).lower()
                method_name = '{}_{}'.format(variable_name, variable_type)
        return method_name

    @property
    def _variable_pattern(self):
        """Regex pattern to match and parse a playbook variable."""
        variable_pattern = r'#([A-Za-z]+)'  # match literal (#App) at beginning of String
        variable_pattern += r':([\d]+)'  # app id (:7979)
        variable_pattern += r':([A-Za-z0-9_\.\-\[\]]+)'  # variable name (:variable_name)
        variable_pattern += r'!(StringArray|BinaryArray|KeyValueArray'  # variable type (array)
        variable_pattern += r'|TCEntityArray|TCEnhancedEntityArray'  # variable type (array)
        variable_pattern += r'|String|Binary|KeyValue|TCEntity|TCEnhancedEntity'  # variable type
        variable_pattern += r'|(?:(?!String)(?!Binary)(?!KeyValue)'  # non matching for custom
        variable_pattern += r'(?!TCEntity)(?!TCEnhancedEntity)'  # non matching for custom
        variable_pattern += r'[A-Za-z0-9_-]+))'  # variable type (custom)
        return variable_pattern

    def generate(self, output_variables):
        """If not currently exist, generates the validation file."""
        validation_file = os.path.join(self.base_dir, 'validate.py')
        if not os.path.isfile(validation_file):
            template = Template(self.validation_template)
            output_data = self.output_data(output_variables)
            template_data = {'output_data': output_data}
            rendered_template = template.render(**template_data)
            with open(validation_file, 'a') as f:
                f.write(rendered_template)

    def generate_test_template(self, variables, feature, file_):
        """If not currently exist, generates the test file file."""
        validation_file = os.path.join(self.base_dir, feature, file_)
        if not os.path.isfile(validation_file):
            template = Template(self.test_template)
            rendered_template = template.render(**variables)
            with open(validation_file, 'a') as f:
                f.write(rendered_template)

    def generate_feature(self, feature, file_):
        """If not currently exist, generates the validation file."""
        validation_file = os.path.join(self.base_dir, feature, 'validate_feature.py')
        if not os.path.isfile(validation_file):
            template = Template(self.validation_feature_template)
            template_data = {'feature': feature, 'file': file_}
            rendered_template = template.render(**template_data)
            with open(validation_file, 'a') as f:
                f.write(rendered_template)

    def output_data(self, output_variables):
        """Return formatted output data.

        #App:9876:http.content!Binary
        """
        output_data = []
        for ov in output_variables:
            output_data.append({'method': self._method_name(ov), 'variable': ov})
        return output_data

    @property
    def validation_template(self):
        """Return template file"""
        url = '{}/{}/app_init/tests/{}'.format(BASE_URL, self.branch, 'validate.py.tpl')
        r = requests.get(url, allow_redirects=True)
        if not r.ok:
            raise RuntimeError('Could not download template file.')
        return r.content

    @property
    def validation_feature_template(self):
        """Return template file"""
        url = '{}/{}/app_init/tests/{}'.format(BASE_URL, self.branch, 'validate_feature.py.tpl')
        r = requests.get(url, allow_redirects=True)
        if not r.ok:
            raise RuntimeError('Could not download template file.')
        return r.content

    @property
    def test_template(self):
        """Return template file"""
        url = '{}/{}/app_init/tests/{}'.format(BASE_URL, self.branch, 'test_template.py.tpl')
        r = requests.get(url, allow_redirects=True)
        if not r.ok:
            raise RuntimeError('Could not download template file.')
        return r.content


class TcExTest(TcExBin):
    """Create profiles for ThreatConnect Job or Playbook App.

    Args:
        _args (namespace): The argparser args Namespace.
    """

    def __init__(self, _args):
        """Initialize Class properties.

        Args:
            _args (namespace): The argparser args Namespace.
        """

        super(TcExTest, self).__init__(_args)

        # properties
        if not self.args.permutations:
            self.base_dir = os.path.join(self.app_path, 'tests')
            self.feature_dir = os.path.join(self.base_dir, self.args.feature)
            self.feature_profile_dir = os.path.join(self.base_dir, self.args.feature, 'profiles.d')
            self.profiles = Profiles(self.profiles_dir)
            self.validation = Validation(self.base_dir, self.args.branch)
            self._output_variables = None

    @staticmethod
    def _print_results(file, status):
        """Print the download results.

        Args:
            file (str): The filename.
            status (str): The file download status.
        """

        file_color = c.Fore.GREEN
        status_color = c.Fore.RED
        if status == 'Success':
            status_color = c.Fore.GREEN
        elif status == 'Skipped':
            status_color = c.Fore.YELLOW
        print(
            '{}{!s:<13}{}{!s:<50}\n{}{!s:<13}{}{}'.format(
                c.Fore.CYAN,
                'Downloading:',
                file_color,
                file,
                c.Fore.CYAN,
                'Status:',
                status_color,
                status,
            )
        )

    def add_profile(self):
        """Add the desired profile"""
        sort_keys = True
        if self.args.profile_file:
            sort_keys = False
            data = []
            profile_file = os.path.join(self.app_path, 'tcex.d', 'profiles', self.args.profile_file)
            if os.path.isfile(self.args.profile_file):
                with open(self.args.profile_file, 'r') as fh:
                    data = json.load(fh)
            elif os.path.isfile(profile_file):
                with open(profile_file, 'r') as fh:
                    data = json.load(fh)
            else:
                self.handle_error(
                    'Error reading in profile file: {}'.format(self.args.profile_file), True
                )

            profile_data = {}
            for d in data:
                profile_data[d.get('profile_name')] = {
                    'exit_codes': d.get('exit_codes'),
                    'inputs': d.get('args', {}).get('app'),
                    'runtime_level': self.ij.runtime_level,
                    'stage': {'redis': self.add_profile_staging(d.get('data_files', []))},
                }

        elif self.args.permutation_id is not None:
            profile_data = {
                self.args.profile_name: {
                    'inputs': {
                        'optional': self.profile_settings_args_layout_json(False),
                        'required': self.profile_settings_args_layout_json(True),
                    },
                    'runtime_level': self.ij.runtime_level,
                }
            }
        elif self.ij.runtime_level.lower() == 'triggerservice':
            profile_data = {
                self.args.profile_name: {
                    'configs': [
                        {
                            'config_id': str(randint(1000, 9999)),
                            'config': self.ij.params_to_args(service_config=False),
                        }
                    ],
                    'runtime_level': self.ij.runtime_level,
                    'trigger': {},
                }
            }
        else:
            profile_data = {
                self.args.profile_name: {
                    'inputs': {
                        'optional': self.ij.params_to_args(required=False),
                        'required': self.ij.params_to_args(required=True),
                    },
                    'runtime_level': self.ij.runtime_level,
                }
            }

        # add profiles
        for profile_name, data in profile_data.items():
            self.profiles.add(profile_name, data, sort_keys=sort_keys)

    @staticmethod
    def add_profile_staging(staging_files):
        """Get existing staging data."""
        staging_data = {}
        for sf in staging_files:
            with open(sf, 'r') as fh:
                data = json.load(fh)
            for d in data:
                staging_data[d.get('variable')] = d.get('data')
        return staging_data

    def create_dirs(self):
        """Create tcex.d directory and sub directories."""
        for d in [self.base_dir, self.feature_dir, self.feature_profile_dir]:
            if not os.path.isdir(d):
                os.makedirs(d)

        # create __init__ files
        self.create_dirs_init()

    def create_dirs_init(self):
        """Create the __init__.py file under dir."""
        for d in [self.base_dir, self.feature_dir]:
            if os.path.isdir(d):
                with open(os.path.join(d, '__init__.py'), 'a'):
                    os.utime(os.path.join(d, '__init__.py'), None)

    def download_file(self, remote_filename):
        """Download file from github.

        Args:
            remote_filename (str): The name of the file as defined in git repository.
        """
        # TODO: can this method be merged with tcex_bin_init download_file?
        status = 'Failed'
        local_filename = self.test_file
        if not os.path.isfile(local_filename):
            url = '{}/{}/app_init/tests/{}'.format(BASE_URL, self.args.branch, remote_filename)
            r = requests.get(url, allow_redirects=True)
            if r.ok:
                open(local_filename, 'wb').write(r.content)
                status = 'Success'
            else:
                self.handle_error('Error requesting: {}'.format(url), False)

            # print download status
            self._print_results(local_filename, status)

    def download_conftest(self):
        """Download conftest.py file from github."""
        # TODO: combine all download methods
        status = 'Failed'
        local_filename = os.path.join('tests', 'conftest.py')
        if not os.path.isfile(local_filename):
            url = '{}/{}/app_init/tests/{}'.format(BASE_URL, self.args.branch, 'conftest.py')
            r = requests.get(url, allow_redirects=True)
            if r.ok:
                open(local_filename, 'wb').write(r.content)
                status = 'Success'
            else:
                self.handle_error('Error requesting: {}'.format(url), False)

            # print download status
            self._print_results(local_filename, status)

    def download_profile(self):
        """Download conftest.py file from github."""
        # TODO: combine all download methods
        status = 'Failed'
        local_filename = os.path.join('tests', 'profiles.py')
        if not os.path.isfile(local_filename):
            url = '{}/{}/app_init/tests/{}'.format(BASE_URL, self.args.branch, 'profiles.py')
            r = requests.get(url, allow_redirects=True)
            if r.ok:
                open(local_filename, 'wb').write(r.content)
                status = 'Success'
            else:
                self.handle_error('Error requesting: {}'.format(url), False)

            # print download status
            self._print_results(local_filename, status)

    def generate_validation_file(self):
        """Generate the validation file."""
        self.validation.generate(self.output_variables)
        self.validation.generate_feature(self.args.feature, self.args.file)

        test_template_variables = {
            'validate_batch_method': '',
            'parent_class': 'TestCasePlaybook',
            'parent_import': 'from tcex.testing import TestCasePlaybook',
        }
        if self.ij.runtime_level.lower() == 'organization':
            # TODO: remove this after validating
            # test_template_variables = {
            #     'validate_batch_method': 'self.validator.threatconnect.batch('
            #     'self.context, '
            #     'pd.get(\'owner\'),'
            #     'pd.get(\'validation_criteria\', {})'
            #     ')',
            #     'parent_class': 'TestCaseJob',
            #     'parent_import': 'from tcex.testing import TestCaseJob',
            # }
            test_template_variables = {
                'validate_batch_method': (
                    'self.validator.threatconnect.batch(self.context, pd.get(\'owner\'),'
                    'pd.get(\'validation_criteria\', {}))'
                ),
                'parent_class': 'TestCaseJob',
                'parent_import': 'from tcex.testing import TestCaseJob',
            }
        self.validation.generate_test_template(
            test_template_variables, self.args.feature, self.test_file
        )

    @property
    def output_variables(self):
        """Return playbook output variables"""
        if self._output_variables is None:
            self._output_variables = []
            # Currently there is no support for projects with multiple install.json files.
            for p in self.ij.playbook.get('outputVariables') or []:
                # "#App:9876:app.data.count!String"
                self._output_variables.append(
                    '#App:{}:{}!{}'.format(9876, p.get('name'), p.get('type'))
                )
        return self._output_variables

    @property
    def permutations_file(self):
        """Return permutations fully qualified filename."""
        return os.path.join(self.base_dir, 'permutations.json')

    def permutation_file_exists(self):
        """Check to see if the permutations file exists"""
        return os.path.isfile(self.permutations_file)

    @property
    def profiles_dir(self):
        """Return profile fully qualified filename."""
        return os.path.join(self.base_dir, self.args.feature, 'profiles.d')

    @property
    def test_file(self):
        """Return generate test filename."""
        test_file = None
        if self.args.file:
            # TODO: add comment on what this does
            if not self.args.file.startswith('test_'):
                self.args.file = 'test_' + self.args.file
            if not self.args.file.endswith('.py'):
                self.args.file = self.args.file + '.py'
            test_file = os.path.join(self.base_dir, self.args.feature, self.args.file)

        return test_file

    @property
    def validation_base_file(self):
        """Return validations fully qualified filename."""
        return os.path.join(self.base_dir, 'validation_base.py')

    @property
    def validation_file(self):
        """Return validations fully qualified filename."""
        return os.path.join(self.base_dir, self.args.feature, 'validation.py')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""TcEx Framework Bin Command Base Module."""
import json
import os
import sys

try:
    import sqlite3
except ModuleNotFoundError:
    # this module is only required for certain CLI commands
    pass

import colorama as c
import redis

from .tcex_install_json import InstallJson
from .tcex_layout_json import LayoutJson


class TcExBin(object):
    """Base Class for ThreatConnect command line tools.

    Args:
        _args (namespace): The argparser args Namespace.
    """

    def __init__(self, _args):
        """Initialize Class properties.

        Args:
            _args (namespace): The argparser args Namespace.
        """
        self.args = _args

        # properties
        self._db_conn = None
        self._input_permutations = []
        self._output_permutations = []
        self._redis = None
        self._tcex_json = None
        self.app_path = os.getcwd()
        self.exit_code = 0
        self.ij = InstallJson('install.json', self.app_path)
        self.lj = LayoutJson('layout.json', self.app_path)
        self.input_table = 'inputs'
        self.output = []

        # initialize colorama
        c.init(autoreset=True, strip=False)

    @staticmethod
    def _to_bool(value):
        """Convert string value to bool."""
        bool_value = False
        if str(value).lower() in ['1', 'true']:
            bool_value = True
        return bool_value

    @property
    def db_conn(self):
        """Create a temporary in memory DB and return the connection."""
        if self._db_conn is None:
            try:
                self._db_conn = sqlite3.connect(':memory:')
            except sqlite3.Error as e:
                self.handle_error(e)
        return self._db_conn

    def db_create_table(self, table_name, columns):
        """Create a temporary DB table.

        Arguments:
            table_name (str): The name of the table.
            columns (list): List of columns to add to the DB.
        """
        formatted_columns = ''
        for col in set(columns):
            formatted_columns += '"{}" text, '.format(col.strip('"').strip('\''))
        formatted_columns = formatted_columns.strip(', ')

        create_table_sql = 'CREATE TABLE IF NOT EXISTS {} ({});'.format(
            table_name, formatted_columns
        )
        try:
            cr = self.db_conn.cursor()
            cr.execute(create_table_sql)
        except sqlite3.Error as e:
            self.handle_error(e)

    def db_insert_record(self, table_name, columns):
        """Insert records into DB.

        Args:
            table_name (str): The name of the table.
            columns (list): List of columns for insert statement.
        """
        bindings = ('?,' * len(columns)).strip(',')
        values = [None] * len(columns)
        sql = 'INSERT INTO {} ({}) VALUES ({})'.format(table_name, ', '.join(columns), bindings)
        cur = self.db_conn.cursor()
        cur.execute(sql, values)

    def db_update_record(self, table_name, column, value):
        """Insert records into DB.

        Args:
            table_name (str): The name of the table.
            column (str): The column name in which the value is to be updated.
            value (str): The value to update in the column.
        """
        sql = 'UPDATE {} SET {} = \'{}\''.format(table_name, column, value)
        cur = self.db_conn.cursor()
        cur.execute(sql)

    def gen_permutations(self, index=0, args=None):
        """Iterate recursively over layout.json parameter names to build permutations.

        .. NOTE:: Permutations are for layout.json based playbook Apps.

        Args:
            index (int, optional): The current index position in the layout names list.
            args (list, optional): Defaults to None. The current list of args.
        """
        if args is None:
            args = []
        try:
            if self.ij.runtime_level.lower() == 'playbook':
                name = self.lj.parameters_names[index]
                display = self.lj.parameters_dict.get(name, {}).get('display')
            else:
                name = list(self.ij.params_dict.keys())[index]
                display = False

            input_type = self.ij.params_dict.get(name, {}).get('type')
            if input_type is None:
                self.handle_error('No value found in install.json for "{}".'.format(name))

            if self.ij.runtime_level.lower() == 'organization' or self.validate_layout_display(
                self.input_table, display
            ):
                if input_type.lower() == 'boolean':
                    for val in [True, False]:
                        args.append({'name': name, 'value': val})
                        self.db_update_record(self.input_table, name, val)
                        self.gen_permutations(index + 1, list(args))
                        # remove the previous arg before next iteration
                        args.pop()
                elif input_type.lower() == 'choice':
                    valid_values = self.ij.expand_valid_values(
                        self.ij.params_dict.get(name, {}).get('validValues', [])
                    )
                    for val in valid_values:
                        args.append({'name': name, 'value': val})
                        self.db_update_record(self.input_table, name, val)
                        self.gen_permutations(index + 1, list(args))
                        # remove the previous arg before next iteration
                        args.pop()
                else:
                    args.append({'name': name, 'value': None})
                    self.gen_permutations(index + 1, list(args))
            else:
                self.gen_permutations(index + 1, list(args))

        except IndexError:
            # when IndexError is reached all data has been processed.
            self._input_permutations.append(args)
            outputs = []

            for o_name in self.ij.output_variables_dict():
                if self.lj.outputs_dict.get(o_name) is not None:
                    display = self.lj.outputs_dict.get(o_name, {}).get('display')
                    valid = self.validate_layout_display(self.input_table, display)
                    if display is None or not valid:
                        continue
                outputs.append(self.ij.output_variables_dict().get(o_name))
            self._output_permutations.append(outputs)

    @staticmethod
    def handle_error(err, halt=True):
        """Print errors message and optionally exit.

        Args:
            err (str): The error message to print.
            halt (bool, optional): Defaults to True. If True the script will exit.
        """
        print('{}{}{}'.format(c.Style.BRIGHT, c.Fore.RED, err))
        if halt:
            sys.exit(1)

    def permutations(self):
        """Process layout.json names/display to get all permutations of args."""
        if 'sqlite3' not in sys.modules:
            print('The sqlite3 module needs to be build-in to Python for this feature.')
            sys.exit(1)

        # create db for permutations testing
        self.db_create_table(self.input_table, self.ij.params_dict.keys())
        self.db_insert_record(self.input_table, self.ij.params_dict.keys())

        # only gen permutations if none have been generated previously
        if not self._input_permutations and not self._output_permutations:
            self.gen_permutations()

        # output permutations
        self.print_permutations()

    def print_permutations(self):
        """Print all valid permutations."""
        index = 0
        permutations = []
        for p in self._input_permutations:
            permutations.append({'index': index, 'args': p})
            index += 1
        with open('permutations.json', 'w') as fh:
            json.dump(permutations, fh, indent=2, sort_keys=True)
        print('All permutations written to the "permutations.json" file.')

    def profile_settings_args_layout_json(self, required):
        """Return args based on layout.json and conditional rendering.

        Args:
            required (bool): If True only required args will be returned.

        Returns:
            dict: Dictionary of required or optional App args.
        """
        self.db_create_table(self.input_table, self.ij.params_dict.keys())
        self.db_insert_record(self.input_table, self.ij.params_dict.keys())

        # only gen permutations if none have been generated previously
        if not self._input_permutations and not self._output_permutations:
            self.gen_permutations()

        profile_args = {}
        try:
            for pn in self._input_permutations[self.args.permutation_id]:
                p = self.ij.filter_params_dict(required=required).get(pn.get('name'))
                if p is None:
                    continue

                if p.get('type').lower() == 'boolean':
                    # use the value generated in the permutation
                    profile_args[p.get('name')] = pn.get('value')
                elif p.get('type').lower() == 'choice':
                    # use the value generated in the permutation
                    profile_args[p.get('name')] = pn.get('value')
                elif p.get('type').lower() == 'keyvaluelist':
                    profile_args[p.get('name')] = '<KeyValueList>'
                elif p.get('name') in ['api_access_id', 'api_secret_key']:
                    # leave these parameters set to the value defined in defaults
                    pass
                else:
                    # add type stub for values
                    types = '|'.join(p.get('playbookDataType', []))
                    if types:
                        profile_args[p.get('name')] = p.get('default', '<{}>'.format(types))
                    else:
                        profile_args[p.get('name')] = p.get('default', '')
        except IndexError:
            self.handle_error('Invalid permutation index provided.')
        return profile_args

    @property
    def redis(self):
        """Return instance of Redis."""
        if self._redis is None:
            self._redis = redis.StrictRedis(host=self.args.redis_host, port=self.args.redis_port)
        return self._redis

    @property
    def tcex_json(self):
        """Return tcex.json file contents."""
        file_fqpn = os.path.join(self.app_path, 'tcex.json')
        if self._tcex_json is None:
            if os.path.isfile(file_fqpn):
                try:
                    with open(file_fqpn, 'r') as fh:
                        self._tcex_json = json.load(fh)
                except ValueError as e:
                    self.handle_error('Failed to load "{}" file ({}).'.format(file_fqpn, e))
            else:
                self.handle_error('File "{}" could not be found.'.format(file_fqpn))
        return self._tcex_json

    @staticmethod
    def update_system_path():
        """Update the system path to ensure project modules and dependencies can be found."""
        cwd = os.getcwd()
        lib_dir = os.path.join(os.getcwd(), 'lib_')
        lib_latest = os.path.join(os.getcwd(), 'lib_latest')

        # insert the lib_latest directory into the system Path if no other lib directory found. This
        # entry will be bumped to index 1 after adding the current working directory.
        if not [p for p in sys.path if lib_dir in p]:
            sys.path.insert(0, lib_latest)

        # insert the current working directory into the system Path for the App, ensuring that it is
        # always the first entry in the list.
        try:
            sys.path.remove(cwd)
        except ValueError:
            pass
        sys.path.insert(0, cwd)

    def validate_layout_display(self, table, display_condition):
        """Check to see if the display condition passes.

        Args:
            table (str): The name of the DB table which hold the App data.
            display_condition (str): The "where" clause of the DB SQL statement.

        Returns:
            bool: True if the row count is greater than 0.
        """
        display = False
        if display_condition is None:
            display = True
        else:
            display_query = 'select count(*) from {} where {}'.format(table, display_condition)
            try:
                cur = self.db_conn.cursor()
                cur.execute(display_query.replace('"', ''))
                rows = cur.fetchall()
                if rows[0][0] > 0:
                    display = True
            except sqlite3.Error as e:
                print('"{}" query returned an error: ({}).'.format(display_query, e))
                sys.exit(1)
        return display



# -*- coding: utf-8 -*-
"""TcEx Playbook Common module"""
import json
import os
from .test_case import TestCase


class TestCasePlaybookCommon(TestCase):
    """Playbook TestCase Class"""

    _output_variables = None
    redis_client = None
    redis_staging_data = {
        '#App:1234:empty!String': '',
        '#App:1234:null!String': None,
        '#App:1234:non-ascii!String': 'ドメイン.テスト',
    }

    @property
    def default_args(self):
        """Return App default args."""
        args = super(TestCasePlaybookCommon, self).default_args
        args.update(
            {
                'tc_playbook_db_context': self.context,
                'tc_playbook_db_path': os.getenv('TC_PLAYBOOK_DB_PATH', 'localhost'),
                'tc_playbook_db_port': os.getenv('TC_PLAYBOOK_DB_PORT', '6379'),
                'tc_playbook_db_type': os.getenv('TC_PLAYBOOK_DB_TYPE', 'Redis'),
                'tc_playbook_out_variables': '',
            }
        )
        return args

    @property
    def output_variables(self):
        """Return playbook output variables"""
        if self._output_variables is None:
            self._output_variables = []
            # Currently there is no support for projects with multiple install.json files.
            for p in self.install_json.get('playbook', {}).get('outputVariables') or []:
                # "#App:9876:app.data.count!String"
                self._output_variables.append(
                    '#App:{}:{}!{}'.format(9876, p.get('name'), p.get('type'))
                )
        return self._output_variables

    def populate_output_variables(self, profile_name):
        """Generate validation rules from App outputs."""
        profile_filename = os.path.join(self.profiles_dir, '{}.json'.format(profile_name))
        with open(profile_filename, 'r+') as fh:
            profile_data = json.load(fh)

            redis_data = self.redis_client.hgetall(self.context)
            outputs = {}
            for variable in self.output_variables:
                data = redis_data.get(variable.encode('utf-8'))

                # validate redis variables
                if data is None:
                    # log warning missing output data
                    self.log.error(
                        '[{}] Missing redis output for variable {}'.format(profile_name, variable)
                    )
                else:
                    data = json.loads(data.decode('utf-8'))

                # validate validation variables
                validation_output = (profile_data.get('outputs') or {}).get(variable)
                if validation_output is None and profile_data.get('outputs') is not None:
                    self.log.error(
                        '[{}] Missing validations rule: {}'.format(profile_name, variable)
                    )
                outputs[variable] = {'expected_output': data, 'op': 'eq'}

            if profile_data.get('outputs') is None:
                # update the profile
                profile_data['outputs'] = outputs

                fh.seek(0)
                fh.write(json.dumps(profile_data, indent=2, sort_keys=True))
                fh.truncate()

    def run(self, args):
        """Implement in Child Class"""
        raise NotImplementedError('Child class must implement this method.')

    def stage_data(self, staged_data):
        """Stage the data in the profile."""
        for key, value in list(staged_data.get('redis', {}).items()):
            self.stager.redis.stage(key, value)



# -*- coding: utf-8 -*-
"""ThreatConnect TI Signature"""
from tcex.tcex_ti.mappings.group.tcex_ti_group import Group


class Signature(Group):
    """Unique API calls for Signature API Endpoints"""

    def __init__(self, tcex, name, file_name, file_type, file_text, owner=None, **kwargs):
        """Initialize Class Properties.

        Valid file_types:
        + Snort ®
        + Suricata
        + YARA
        + ClamAV ®
        + OpenIOC
        + CybOX ™
        + Bro
        + Regex
        + SPL - Splunk ® Search Processing Language

        Args:
            name (str): The name for this Group.
            file_name (str): The name for the attached signature for this Group.
            file_type (str): The signature type for this Group.
            file_text (str): The signature content for this Group.
            date_added (str, kwargs): The date timestamp the Indicator was created.
        """
        super(Signature, self).__init__(
            tcex, 'Signature', 'signature', 'signatures', owner=owner, name=name, **kwargs
        )
        self._data['fileName'] = file_name
        self._data['fileType'] = file_type
        self._data['fileText'] = file_text

    def download(self):
        """
        Downloads the signature.

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        return self.tc_requests.download(self.api_type, self.api_branch, self.unique_id)



# -*- coding: utf-8 -*-
"""Base pytest configuration file."""
import os
import shutil


# clear log directory
def clear_log_directory():
    """Clear the App log directory."""
    log_directory = 'log'
    if os.path.isdir(log_directory):
        print('Clearing log directory.')
        for log_file in os.listdir(log_directory):
            file_path = os.path.join(log_directory, log_file)
            if os.path.isdir(file_path):
                shutil.rmtree(file_path)
            if os.path.isfile(file_path):
                os.remove(file_path)


clear_log_directory()



# -*- coding: utf-8 -*-
"""Test the TcEx Batch Module."""

import pytest

from ..tcex_init import tcex


# pylint: disable=R0201,W0201
class TestGroup3:
    """Test the TcEx Batch Module."""

    def setup_class(self):
        """Configure setup before all tests."""

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-adversary-i3-001', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('pytest-adversary-i3-002', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('pytest-adversary-i3-003', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('pytest-adversary-i3-004', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_adversary(self, name, description, label, tag):
        """Test adversary creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'adversary', name])
        ti = batch.add_group(
            {
                'name': name,
                'type': 'Adversary',
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-campaign-i3-001', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('pytest-campaign-i3-002', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('pytest-campaign-i3-003', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('pytest-campaign-i3-004', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_campaign(self, name, description, label, tag):
        """Test campaign creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'campaign', name])
        ti = batch.add_group(
            {
                'name': name,
                'type': 'Campaign',
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-document-i3-001', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('pytest-document-i3-002', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('pytest-document-i3-003', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('pytest-document-i3-004', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_document(self, name, description, label, tag):
        """Test document creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'document', name])
        ti = batch.add_group(
            {
                'name': name,
                'fileName': 'test.txt',
                'fileContent': 'Example file content',
                'type': 'Document',
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-document-malware-i3-001', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('pytest-document-malware-i3-002', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('pytest-document-malware-i3-003', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('pytest-document-malware-i3-004', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_document_malware(self, name, description, label, tag):
        """Test document creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'document', name])
        ti = batch.add_group(
            {
                'name': name,
                'fileName': 'test.zip',
                'fileContent': 'Example file content',
                'malware': True,
                'password': 'test',
                'type': 'Document',
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-email-i3-001', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('pytest-email-i3-002', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('pytest-email-i3-003', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('pytest-email-i3-004', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_email(self, name, description, label, tag):
        """Test email creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'email', name])
        ti = batch.add_group(
            {
                'name': name,
                'body': 'Email Body',
                'header': 'Email Header',
                'subject': 'Email Subject',
                'type': 'Email',
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-event-i3-001', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('pytest-event-i3-002', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('pytest-event-i3-003', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('pytest-event-i3-004', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_event(self, name, description, label, tag):
        """Test event creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'event', name])
        ti = batch.add_group(
            {
                'name': name,
                'type': 'Event',
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-incident-i3-001', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('pytest-incident-i3-002', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('pytest-incident-i3-003', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('pytest-incident-i3-004', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_incident(self, name, description, label, tag):
        """Test incident creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'incident', name])
        ti = batch.add_group(
            {
                'name': name,
                'type': 'Incident',
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-intrusion_set-i3-001', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('pytest-intrusion_set-i3-002', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('pytest-intrusion_set-i3-003', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('pytest-intrusion_set-i3-004', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_intrusion_set(self, name, description, label, tag):
        """Test intrusion_set creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'intrusion_set', name])
        ti = batch.add_group(
            {
                'name': name,
                'type': 'Intrusion Set',
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-report-i3-001', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('pytest-report-i3-002', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('pytest-report-i3-003', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('pytest-report-i3-004', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_report(self, name, description, label, tag):
        """Test report creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'report', name])
        ti = batch.add_group(
            {
                'name': name,
                'fileContent': 'Report Content',
                'fileName': 'test.pdf',
                'type': 'Report',
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-signature-i3-001', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('pytest-signature-i3-002', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('pytest-signature-i3-003', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('pytest-signature-i3-004', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_signature(self, name, description, label, tag):
        """Test signature creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'signature', name])
        ti = batch.add_group(
            {
                'name': name,
                'fileName': 'test.snort',
                'fileType': 'Snort',
                'fileText': 'Signature content',
                'type': 'Signature',
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-threat-i3-001', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('pytest-threat-i3-002', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('pytest-threat-i3-003', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('pytest-threat-i3-004', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_threat(self, name, description, label, tag):
        """Test threat creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'threat', name])
        ti = batch.add_group(
            {
                'name': name,
                'type': 'Threat',
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1



# -*- coding: utf-8 -*-
"""Test the TcEx Batch Module."""

import pytest

from ..tcex_init import tcex


# pylint: disable=R0201,W0201
class TestIndicator2:
    """Test the TcEx Batch Module."""

    def setup_class(self):
        """Configure setup before all tests."""

    @pytest.mark.parametrize(
        'indicator,description,label,tag',
        [
            ('2.22.222.1', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('2.22.222.2', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('2.22.222.3', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('2.22.222.4', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_address(self, indicator, description, label, tag):
        """Test address creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'address', indicator])
        ti = batch.indicator(
            indicator_type='Address', summary=indicator, rating='5.0', confidence='100', xid=xid
        )
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'indicator,description,label,tag',
        [
            ('pytest-email_address-i2-001@test.com', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-email_address-i2-002@test.com', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-email_address-i2-003@test.com', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-email_address-i2-004@test.com', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_email_address(self, indicator, description, label, tag):
        """Test email_address creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'email_address', indicator])
        ti = batch.indicator(
            indicator_type='EmailAddress',
            summary=indicator,
            rating='5.0',
            confidence='100',
            xid=xid,
        )
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'md5,sha1,sha256,description,label,tag',
        [
            ('a2', 'a2', 'a2', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('b2', 'b2', 'b2', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('c2', 'c2', 'c2', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('d2', 'd2', 'd2', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_file(self, md5, sha1, sha256, description, label, tag):
        """Test file creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'file', md5, sha1, sha256])
        ti = batch.indicator(
            indicator_type='File',
            summary='{} : {} : {}'.format(md5 * 16, sha1 * 20, sha256 * 32),
            rating='5.0',
            confidence='100',
            xid=xid,
        )
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'indicator,description,label,tag',
        [
            ('pytest-host-i2-001.com', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-host-i2-002.com', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-host-i2-003.com', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-host-i2-004.com', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_host(self, indicator, description, label, tag):
        """Test host creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'host', indicator])
        ti = batch.indicator(
            indicator_type='Host', summary=indicator, rating='5.0', confidence='100', xid=xid
        )
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'indicator,description,label,tag',
        [
            ('https://pytest-url-i2-001.com', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('https://pytest-url-i2-002.com', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('https://pytest-url-i2-003.com', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('https://pytest-url-i2-004.com', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_url(self, indicator, description, label, tag):
        """Test url creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'url', indicator])
        ti = batch.indicator(
            indicator_type='URL', summary=indicator, rating='5.0', confidence='100', xid=xid
        )
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1



# -*- coding: utf-8 -*-
"""Test the TcEx Batch Module."""

import pytest

from ..tcex_init import tcex


# pylint: disable=R0201,W0201
class TestIndicator3:
    """Test the TcEx Batch Module."""

    def setup_class(self):
        """Configure setup before all tests."""

    @pytest.mark.parametrize(
        'indicator,description,label,tag',
        [
            ('3.33.33.1', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('3.33.33.2', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('3.33.33.3', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('3.33.33.4', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_address(self, indicator, description, label, tag):
        """Test address creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'address', indicator])
        ti = batch.add_indicator(
            {
                'type': 'Address',
                'rating': 5.00,
                'confidence': 100,
                'summary': indicator,
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'indicator,description,label,tag',
        [
            ('pytest-email_address-i3-001@test.com', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-email_address-i3-002@test.com', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-email_address-i3-003@test.com', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-email_address-i3-004@test.com', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_email_address(self, indicator, description, label, tag):
        """Test email_address creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'email_address', indicator])
        ti = batch.add_indicator(
            {
                'type': 'EmailAddress',
                'rating': 5.00,
                'confidence': 100,
                'summary': indicator,
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'md5,sha1,sha256,description,label,tag',
        [
            ('a3', 'a3', 'a3', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('b3', 'b3', 'b3', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('c3', 'c3', 'c3', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('d3', 'd3', 'd3', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_file(self, md5, sha1, sha256, description, label, tag):
        """Test file creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'file', md5, sha1, sha256])
        ti = batch.add_indicator(
            {
                'type': 'File',
                'rating': 5.00,
                'confidence': 100,
                'summary': '{} : {} : {}'.format(md5 * 16, sha1 * 20, sha256 * 32),
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'indicator,description,label,tag',
        [
            ('pytest-host-i3-001.com', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-host-i3-002.com', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-host-i3-003.com', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-host-i3-004.com', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_host(self, indicator, description, label, tag):
        """Test host creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'host', indicator])
        ti = batch.add_indicator(
            {
                'type': 'Host',
                'rating': 5.00,
                'confidence': 100,
                'summary': indicator,
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'indicator,description,label,tag',
        [
            ('https://pytest-url-i3-001.com', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('https://pytest-url-i3-002.com', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('https://pytest-url-i3-003.com', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('https://pytest-url-i3-004.com', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_url(self, indicator, description, label, tag):
        """Test url creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'url', indicator])
        ti = batch.add_indicator(
            {
                'type': 'Url',
                'rating': 5.00,
                'confidence': 100,
                'summary': indicator,
                'xid': xid,
                'attribute': [{'displayed': True, 'type': 'Description', 'value': description}],
                'securityLabel': [
                    {'color': 'ffc0cb', 'name': label, 'description': 'Pytest Label Description'}
                ],
                'tag': [{'name': tag}],
            }
        )
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1



# -*- coding: utf-8 -*-
"""Stage Data Testing Module"""
import base64
import binascii
import sys
import json
import os


# pylint: disable=R0201
class Stager(object):
    """Stage Data class"""

    def __init__(self, tcex, log, log_data):
        """Initialize class properties"""
        # self.args = tcex.args  # required for args to be parsed
        self.log = log
        self.log_data = log_data
        self.tcex = tcex
        # TODO: validate this
        self.tcex.logger.update_handler_level('error')

        # properties
        self._redis = None
        self._threatconnect = None

    @property
    def redis(self):
        """Get the current instance of Redis for staging data"""
        if not self._redis:
            self._redis = Redis(self)
        return self._redis

    @property
    def threatconnect(self):
        """Get the current instance of ThreatConnect for staging data"""
        if not self._threatconnect:
            self._threatconnect = ThreatConnect(self)
        return self._threatconnect


class Redis(object):
    """Stages the Redis Data"""

    def __init__(self, provider):
        """Initialize class properties."""
        self.provider = provider
        self.redis_client = provider.tcex.playbook.db.r
        self.log_data = self.provider.log_data

    def from_dict(self, staging_data):
        """Stage redis data from dict"""
        for variable, data in staging_data.items():
            variable_type = self.provider.tcex.playbook.variable_type(variable)

            self.log_data('stage', 'variable', variable)
            self.log_data('stage', 'data', data)
            if variable_type == 'Binary':
                data = self._decode_binary(data, variable)
            elif variable_type == 'BinaryArray':
                data = [self._decode_binary(d, variable) for d in data]
            self.provider.tcex.playbook.create(variable, data)

    def stage(self, variable, data):
        """Stage data in redis"""
        self.provider.tcex.playbook.create(variable, data)

    def delete_context(self, context):
        """Delete data in redis"""
        keys = self.redis_client.hkeys(context)
        if keys:
            return self.redis_client.hdel(context, *keys)
        return 0

    @staticmethod
    def _decode_binary(binary_data, variable):
        """Base64 decode binary data."""
        try:
            data = base64.b64decode(binary_data)
        except binascii.Error:
            print(
                'The Binary staging data for variable {} is not properly base64 '
                'encoded.'.format(variable)
            )
            sys.exit()
        return data


class ThreatConnect(object):
    """Stages the ThreatConnect Data"""

    def __init__(self, provider):
        self.provider = provider
        self.batch = None

    def dir(self, directory, owner, batch=False):
        """Stages the directory"""
        entities = []
        for stage_file in os.listdir(directory):
            if not (stage_file.endswith('.json') and stage_file.startswith('tc_stage_')):
                continue
            entities.append(self._convert_to_entities('{}/{}'.format(directory, stage_file)))
        return self.entities(entities, owner, batch=batch)

    def file(self, file, owner, batch=False):
        """Stages the file"""
        entities = self._convert_to_entities(file)
        return self.entities(entities, owner, batch=batch)

    def entities(self, entities, owner, batch=False):
        """Stages all of the provided entities"""
        response = []
        if batch:
            self.batch = self.provider.tcex.batch(owner)
            for entity in entities:
                labels = entity.pop('securityLabels')
                attributes = entity.pop('attributes')
                tags = entity.pop('tags')
                associations = entity.pop('associations')
                entity['xid'] = entity.get(
                    'xid',
                    self.provider.batch.generate_xid(
                        [owner, entity.get('type'), entity.get('summary')]
                    ),
                )
                batch_entity = self._convert_to_batch_entity(entity)
                self.provider.batch.save(batch_entity)
                for tag in tags:
                    batch_entity.tag(tag)
                for label in labels:
                    batch_entity.label(label)
                for association in associations:
                    batch_entity.association(association)
                for attribute in attributes:
                    batch_entity.attribute(
                        attribute.get('type'),
                        attribute.get('value'),
                        attribute.get('displayed', True),
                    )

            response = self.batch.submit_all()
        else:
            for entity in entities:
                response.append(self.entity(entity, owner))
        return response if not batch else None

    def entity(self, entity, owner):
        """Stage data in ThreatConnect"""
        if not isinstance(entity, list):
            entity = [entity]
        self.provider.tcex.ti.create_entities(entity, owner)

    def clear(self, owner):
        """delete and recreate the owner"""

    @staticmethod
    def _convert_to_entities(file):
        """Convert A file to TC Entity's"""
        with open(file, 'r') as read_file:
            data = json.load(read_file)
        return data

    def _convert_to_batch_entity(self, entity):
        """Convert TC Entity to a Batch entity"""
        entity_type = entity.pop('type')
        ti = None
        if entity_type in self.provider.tcex.indicator_types:
            ti = self.provider.batch.indicator(entity_type, entity.pop('summary'), entity)
        elif entity_type in self.provider.tcex.group_types:
            ti = self.provider.batch.group(entity_type, entity.pop('summary'), entity)
        return ti



# -*- coding: utf-8 -*-
"""Test the TcEx Args Config Module."""

from ..tcex_init import tcex


# pylint: disable=W0201
class TestArgsConfig:
    """Test TcEx Args Config."""

    @staticmethod
    def test_address_get():
        """Test args."""
        assert tcex.args.tc_token
        assert tcex.args.tc_token_expires
        assert tcex.args.api_default_org == 'TCI'



# -*- coding: utf-8 -*-
"""Test the TcEx Threat Intel Module."""

from ..tcex_init import tcex


# pylint: disable=W0201
class TestEmailAddressIndicators:
    """Test TcEx Host Indicators."""

    def setup_class(self):
        """Configure setup before all tests."""
        self.ti = tcex.ti

    def test_email_address_get(self, address='email_address_42353@gmail.com'):
        """Test email_address get."""
        # create
        self.email_address_create(address)

        # get
        ti = self.ti.email_address(address, owner=tcex.args.tc_owner)
        r = ti.single()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get(ti.api_entity).get('address') == address

        # delete
        self.email_address_delete(address)

    def test_email_address_get_attributes(self, address='email_address_12453@gmail.com'):
        """Test email_address get."""
        # create
        self.email_address_create(address)
        self.test_email_address_add_attribute(False, address, 'Description', 'test1')
        self.test_email_address_add_attribute(False, address, 'Description', 'test2')
        self.test_email_address_add_attribute(False, address, 'Description', 'test3')

        # get attributes
        ti = self.ti.email_address(address, owner=tcex.args.tc_owner)
        for attribute in ti.attributes():
            assert attribute
            break
        else:
            assert False

        # delete
        self.email_address_delete(address)

    def test_email_address_get_tags(self, address='email_address_64235@gmail.com'):
        """Test email_address get."""
        # create
        self.email_address_create(address)
        self.test_email_address_add_tag(False, address, 'One')
        self.test_email_address_add_tag(False, address, 'Two')

        # get tags
        ti = self.ti.email_address(address, owner=tcex.args.tc_owner)
        for tag in ti.tags():
            assert tag.get('name')
            break
        else:
            assert False

        # delete
        self.email_address_delete(address)

    def test_email_address_get_include(self, address='email_address_78159@gmail.com'):
        """Test email_address get."""
        self.email_address_create(address)
        self.test_email_address_add_attribute(False, address, 'Description', 'test123')
        self.test_email_address_add_label(False, address, 'TLP:RED')
        self.test_email_address_add_tag(False, address, 'PyTest')

        parameters = {'includes': ['additional', 'attributes', 'labels', 'tags']}
        ti = self.ti.email_address(address, owner=tcex.args.tc_owner)
        r = ti.single(params=parameters)
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('emailAddress').get('address') == address
        assert ti_data.get('data').get('emailAddress').get('attribute')[0].get('value') == 'test123'
        assert (
            ti_data.get('data').get('emailAddress').get('securityLabel')[0].get('name') == 'TLP:RED'
        )
        assert ti_data.get('data').get('emailAddress').get('tag')[0].get('name') == 'PyTest'

        # delete
        self.email_address_delete(address)

    def email_address_create(self, address='email_address_65341@gmail.com'):
        """Test email_address create."""
        ti = self.ti.email_address(address, owner=tcex.args.tc_owner)
        r = ti.create()
        ti_data = r.json()
        assert r.status_code == 201
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('emailAddress').get('address') == address

    def test_email_address_add_attribute(
        self,
        should_create=True,
        address='email_address_nkjvb@gmail.com',
        attribute_type='Description',
        attribute_value='Example Description.',
    ):
        """Test email_address attribute add."""
        if should_create:
            self.email_address_create(address)

        ti = self.ti.email_address(address, owner=tcex.args.tc_owner)
        r = ti.add_attribute(attribute_type=attribute_type, attribute_value=attribute_value)
        attribute_data = r.json()
        assert r.status_code == 201
        assert attribute_data.get('status') == 'Success'
        assert attribute_data.get('data').get('attribute').get('value') == attribute_value
        if should_create:
            self.email_address_delete(address)

    def test_email_address_add_label(
        self, should_create=True, address='email_address_ds4vb@gmail.com', label='TLP:GREEN'
    ):
        """Test email_address attribute add."""
        if should_create:
            self.email_address_create(address)

        ti = self.ti.email_address(address, owner=tcex.args.tc_owner)
        r = ti.add_label(label=label)
        label_data = r.json()
        assert r.status_code == 201
        assert label_data.get('status') == 'Success'
        if should_create:
            self.email_address_delete(address)

    def test_email_address_add_tag(
        self, should_create=True, address='email_address_fdsv23@gmail.com', name='Crimeware'
    ):
        """Test email_address attribute add."""
        if should_create:
            self.email_address_create(address)

        ti = self.ti.email_address(address, owner=tcex.args.tc_owner)
        r = ti.add_tag(name=name)
        tag_data = r.json()
        assert r.status_code == 201
        assert tag_data.get('status') == 'Success'
        if should_create:
            self.email_address_delete(address)

    def email_address_delete(self, address='email_address_523fa@gmail.com'):
        """Test email_address delete."""
        # create indicator
        self.email_address_create(address)

        # delete indicator
        ti = self.ti.email_address(address, owner=tcex.args.tc_owner)
        r = ti.delete()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'

    def test_email_address_update(self, address='email_address_b3da3@gmail.com'):
        """Test email_address update."""
        # create indicator
        self.email_address_create(address)

        # update indicator
        ti = self.ti.email_address(address, rating=5, confidence=10, owner=tcex.args.tc_owner)
        r = ti.update()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('emailAddress').get('rating') == 5.0
        assert ti_data.get('data').get('emailAddress').get('confidence') == 10

        # delete indicator
        self.email_address_delete(address)



# -*- coding: utf-8 -*-
"""Test the TcEx Batch Module."""

import pytest

from ..tcex_init import tcex


# pylint: disable=R0201,W0201
class TestGroup1:
    """Test the TcEx Batch Module."""

    def setup_class(self):
        """Configure setup before all tests."""

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-adversary-i1-001', 'Example #1', 'PYTEST1', 'PyTest1'),
            ('pytest-adversary-i1-002', 'Example #2', 'PYTEST2', 'PyTest2'),
            ('pytest-adversary-i1-003', 'Example #3', 'PYTEST3', 'PyTest3'),
            ('pytest-adversary-i1-004', 'Example #4', 'PYTEST4', 'PyTest4'),
        ],
    )
    def test_adversary(self, name, description, label, tag):
        """Test adversary creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'adversary', name])
        ti = batch.adversary(name=name, xid=xid)
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-campaign-i1-001', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-campaign-i1-002', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-campaign-i1-003', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-campaign-i1-004', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_campaign(self, name, description, label, tag):
        """Test campaign creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'campaign', name])
        ti = batch.campaign(name=name, xid=xid)
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-document-i1-001', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-document-i1-002', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-document-i1-003', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-document-i1-004', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_document(self, name, description, label, tag):
        """Test document creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'document', name])
        ti = batch.document(name=name, file_name='example.txt', xid=xid)
        ti.file_content = 'Example file content'
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-document-malware-i1-001', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-document-malware-i1-002', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-document-malware-i1-003', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-document-malware-i1-004', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_document_malware(self, name, description, label, tag):
        """Test document creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'document', name])
        ti = batch.document(name=name, file_name='example.zip', xid=xid)
        ti.malware = True
        ti.password = 'test'
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-email-i1-001', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-email-i1-002', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-email-i1-003', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-email-i1-004', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_email(self, name, description, label, tag):
        """Test email creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'email', name])
        ti = batch.email(
            name=name, subject='subject', header='test', body='this is just a test', xid=xid
        )
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-event-i1-001', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-event-i1-002', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-event-i1-003', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-event-i1-004', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_event(self, name, description, label, tag):
        """Test event creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'event', name])
        ti = batch.event(name=name, event_data='2008-12-12T12:12:12Z', xid=xid)
        ti.status = 'Escalated'
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-incident-i1-001', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-incident-i1-002', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-incident-i1-003', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-incident-i1-004', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_incident(self, name, description, label, tag):
        """Test incident creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'incident', name])
        ti = batch.incident(name=name, xid=xid)
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-intrusion_set-i1-001', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-intrusion_set-i1-002', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-intrusion_set-i1-003', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-intrusion_set-i1-004', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_intrusion_set(self, name, description, label, tag):
        """Test intrusion_set creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'intrusion_set', name])
        ti = batch.intrusion_set(name=name, xid=xid)
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-report-i1-001', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-report-i1-002', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-report-i1-003', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-report-i1-004', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_report(self, name, description, label, tag):
        """Test report creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'report', name])
        ti = batch.report(name=name, file_name='report.pdf', publish_date='12-12-2008', xid=xid)
        ti.file_content = 'test 123'
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-signature-i1-001', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-signature-i1-002', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-signature-i1-003', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-signature-i1-004', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_signature(self, name, description, label, tag):
        """Test signature creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'signature', name])
        ti = batch.signature(
            name=name, file_name='test.snort', file_type='snort', file_text='test', xid=xid
        )
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1

    @pytest.mark.parametrize(
        'name,description,label,tag',
        [
            ('pytest-threat-i1-001', 'Example #1', 'PYTEST:1', 'PyTest1'),
            ('pytest-threat-i1-002', 'Example #2', 'PYTEST:2', 'PyTest2'),
            ('pytest-threat-i1-003', 'Example #3', 'PYTEST:3', 'PyTest3'),
            ('pytest-threat-i1-004', 'Example #4', 'PYTEST:4', 'PyTest4'),
        ],
    )
    def test_threat(self, name, description, label, tag):
        """Test threat creation"""
        batch = tcex.batch(owner='TCI')
        xid = batch.generate_xid(['pytest', 'threat', name])
        ti = batch.threat(name=name, xid=xid)
        ti.attribute(attr_type='Description', attr_value=description, displayed=True)
        ti.security_label(name=label, description='Pytest Label Description', color='ffc0cb')
        ti.tag(name=tag)
        batch.save(ti)
        batch_status = batch.submit_all()
        assert batch_status[0].get('status') == 'Completed'
        assert batch_status[0].get('successCount') == 1



# -*- coding: utf-8 -*-
"""Test the TcEx Threat Intel Module."""

from ..tcex_init import tcex


# pylint: disable=W0201
class TestEmailGroups:
    """Test TcEx Host Groups."""

    def setup_class(self):
        """Configure setup before all tests."""
        self.ti = tcex.ti

    def test_email_get(
        self,
        name='email-name-42353',
        to='email-to-asdf@gmail.com',
        from_addr='email-from-fdsav@gmail.com',
        subject='email-subject-bfd21',
        body='email-body-fdsab',
        header='email-header-bfd32r',
    ):
        """Test email get."""
        # create
        email_id = self.email_create(name, to, from_addr, subject, body, header)

        # get
        ti = self.ti.email(
            name, to, from_addr, subject, body, header, owner=tcex.args.tc_owner, unique_id=email_id
        )  # pylint: disable=E1121
        r = ti.single()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get(ti.api_entity).get('name') == name

        # delete
        self.email_delete(email_id)

    def test_email_get_attributes(
        self,
        email_id=None,
        name='email-name-12453',
        to='email-to-asdf@gmail.com',
        from_addr='email-from-fdsav@gmail.com',
        subject='email-subject-bfd21',
        body='email-body-fdsab',
        header='email-header-bfd32r',
    ):
        """Test email get."""
        # create
        if not email_id:
            email_id = self.email_create(name, to, from_addr, subject, body, header)
        self.test_email_add_attribute(
            email_id=email_id, attribute_type='Description', attribute_value='test1'
        )
        self.test_email_add_attribute(
            email_id=email_id, attribute_type='Description', attribute_value='test2'
        )
        self.test_email_add_attribute(
            email_id=email_id, attribute_type='Description', attribute_value='test3'
        )

        # get attributes
        ti = self.ti.email(
            name, to, from_addr, subject, body, header, owner=tcex.args.tc_owner, unique_id=email_id
        )
        for attribute in ti.attributes():
            assert attribute
            break
        else:
            assert False

        # delete
        self.email_delete(email_id)

    def test_email_get_tags(
        self,
        email_id=None,
        name='email-name-64235',
        to='email-to-asdf@gmail.com',
        from_addr='email-from-fdsav@gmail.com',
        subject='email-subject-bfd21',
        body='email-body-fdsab',
        header='email-header-bfd32r',
    ):
        """Test email get."""
        # create
        if not email_id:
            email_id = self.email_create(name, to, from_addr, subject, body, header)
        self.test_email_add_tag(email_id=email_id, tag='One')
        self.test_email_add_tag(email_id=email_id, tag='Two')

        # get tags
        ti = self.ti.email(
            name, to, from_addr, subject, body, header, owner=tcex.args.tc_owner, unique_id=email_id
        )
        for tag in ti.tags():
            assert tag.get('name')
            break
        else:
            assert False

        # delete
        self.email_delete(email_id)

    def test_email_get_include(
        self,
        name='email-name-78159',
        to='email-to-fdsabv@gmail.com',
        from_addr='email-from-gfasa@gmail.com',
        subject='email-subject-fds32g',
        body='email-body-165few',
        header='email-header-fdy453',
    ):
        """Test email get."""
        email_id = self.email_create(name, to, from_addr, subject, body, header)
        self.test_email_add_attribute(
            email_id=email_id, attribute_type='Description', attribute_value='test123'
        )
        self.test_email_add_label(email_id=email_id, label='TLP:RED')
        self.test_email_add_tag(email_id=email_id, tag='PyTest')

        parameters = {'includes': ['additional', 'attributes', 'labels', 'tags']}
        ti = self.ti.email(
            name, to, from_addr, subject, body, header, owner=tcex.args.tc_owner, unique_id=email_id
        )
        r = ti.single(params=parameters)
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('email').get('name') == name
        assert ti_data.get('data').get('email').get('attribute')[0].get('value') == 'test123'
        assert ti_data.get('data').get('email').get('securityLabel')[0].get('name') == 'TLP:RED'
        assert ti_data.get('data').get('email').get('tag')[0].get('name') == 'PyTest'

        # delete
        self.email_delete(email_id)

    def email_create(
        self,
        name='email-name-65341',
        to='email-to-t42gad@gmail.com',
        from_addr='email-from-bfds3@gmail.com',
        subject='email-subject-fdsab2',
        body='email-body-fd23fb',
        header='email-header-32fsa',
    ):
        """Test email create."""
        ti = self.ti.email(name, to, from_addr, subject, body, header, owner=tcex.args.tc_owner)
        r = ti.create()
        ti_data = r.json()
        assert r.status_code == 201
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('email').get('name') == name
        return ti.unique_id

    def test_email_add_attribute(
        self,
        name='email-name-fdsab5',
        to='email-to-fdsabv@gmail.com',
        from_addr='email-from-gfasa@gmail.com',
        subject='email-subject-fds32g',
        body='email-body-165few',
        header='email-header-fdy453',
        email_id=None,
        attribute_type='Description',
        attribute_value='Example Description.',
    ):
        """Test email attribute add."""
        should_delete = False
        if not email_id:
            should_delete = True
            email_id = self.email_create(name, to, from_addr, subject, body, header)

        ti = self.ti.email(
            None, None, None, None, None, None, owner=tcex.args.tc_owner, unique_id=email_id
        )
        r = ti.add_attribute(attribute_type=attribute_type, attribute_value=attribute_value)
        attribute_data = r.json()
        assert r.status_code == 201
        assert attribute_data.get('status') == 'Success'
        assert attribute_data.get('data').get('attribute').get('value') == attribute_value

        if should_delete:
            self.email_delete(email_id)

    def test_email_add_label(
        self,
        email_id=None,
        name='email-name-ds4vb',
        to='email-to-fdsabv@gmail.com',
        from_addr='email-from-gfasa@gmail.com',
        subject='email-subject-fds32g',
        body='email-body-165few',
        header='email-header-fdy453',
        label='TLP:GREEN',
    ):
        """Test email attribute add."""
        should_delete = False
        if not email_id:
            should_delete = True
            email_id = self.email_create(name, to, from_addr, subject, body, header)

        ti = self.ti.email(
            name, to, from_addr, subject, body, header, owner=tcex.args.tc_owner, unique_id=email_id
        )
        r = ti.add_label(label=label)
        label_data = r.json()
        assert r.status_code == 201
        assert label_data.get('status') == 'Success'

        if should_delete:
            self.email_delete(email_id)

    def test_email_add_tag(
        self,
        email_id=None,
        name='email-name-fdsv23',
        to='email-to-fdsabv@gmail.com',
        from_addr='email-from-gfasa@gmail.com',
        subject='email-subject-fds32g',
        body='email-body-165few',
        header='email-header-fdy453',
        tag='Crimeware',
    ):
        """Test email attribute add."""
        should_delete = False
        if not email_id:
            should_delete = True
            email_id = self.email_create(name, to, from_addr, subject, body, header)

        ti = self.ti.email(
            name, to, from_addr, subject, body, header, owner=tcex.args.tc_owner, unique_id=email_id
        )
        r = ti.add_tag(tag)
        tag_data = r.json()
        assert r.status_code == 201
        assert tag_data.get('status') == 'Success'

        if should_delete:
            self.email_delete(email_id)

    def email_delete(
        self,
        email_id=None,
        name='email-name-bdsfd',
        to='email-to-fdsabv@gmail.com',
        from_addr='email-from-gfasa@gmail.com',
        subject='email-subject-fds32g',
        body='email-body-165few',
        header='email-header-fdy453',
    ):
        """Test email delete."""
        # create indicator
        if not email_id:
            email_id = self.email_create(name, to, from_addr, subject, body, header)

        # delete indicator
        ti = self.ti.email(
            name, to, from_addr, subject, body, header, owner=tcex.args.tc_owner, unique_id=email_id
        )
        r = ti.delete()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'

    def test_email_update(
        self,
        email_id=None,
        name='email-name-b3da3',
        to='email-to-fdsabv@gmail.com',
        from_addr='email-from-gfasa@gmail.com',
        subject='email-subject-fds32g',
        body='email-body-165few',
        header='email-header-fdy453',
    ):
        """Test email update."""
        # create indicator
        if not email_id:
            email_id = self.email_create(name, to, from_addr, subject, body, header)

        # update indicator
        name = 'email-name-cj98fdsa'
        to = 'email-to-bfdsa@gmail.com'
        from_addr = 'email-from-bfdsa@gmail.com'
        subject = 'email-subject-j980as'
        body = 'email-body-fdsat3'
        header = 'email-header-jk23gf'
        ti = self.ti.email(
            name, to, from_addr, subject, body, header, owner=tcex.args.tc_owner, unique_id=email_id
        )
        r = ti.update()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('email').get('name') == name
        assert ti_data.get('data').get('email').get('from') == from_addr
        assert ti_data.get('data').get('email').get('subject') == subject
        assert ti_data.get('data').get('email').get('body') == body
        assert ti_data.get('data').get('email').get('header') == header

        # delete indicator
        self.email_delete(email_id)



# -*- coding: utf-8 -*-
"""ThreatConnect TI Email """
from tcex.tcex_ti.mappings.group.tcex_ti_group import Group


class Email(Group):
    """Unique API calls for Email API Endpoints"""

    def __init__(self, tcex, name, to, from_addr, subject, body, header, owner=None, **kwargs):
        """Initialize Class Properties.

        Args:
            name (str): The name for this Group.
            subject (str): The subject for this Email.
            header (str): The header for this Email.
            body (str): The body for this Email.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            from_addr (str, kwargs): The **from** address for this Email.
            to (str, kwargs): The **to** address for this Email.
        """
        super(Email, self).__init__(
            tcex, 'Email', 'email', 'emails', owner=owner, name=name, **kwargs
        )
        self._data['to'] = to or kwargs.get('to')
        self._data['from'] = from_addr or kwargs.get('from_addr')
        self._data['subject'] = subject or kwargs.get('subject')
        self._data['body'] = body or kwargs.get('body')
        self._data['header'] = header or kwargs.get('header')
        self._data['score'] = kwargs.get('score', 0)



# -*- coding: utf-8 -*-
"""Validate Data Testing Module"""
import difflib
import hashlib
import json
import operator
import os
import re
import math
import random
from six import string_types

try:
    from urllib import unquote  # Python 2
except ImportError:
    from urllib.parse import unquote  # Python


class Validator(object):
    """Validator"""

    def __init__(self, tcex, log, log_data):
        """Initialize class properties."""
        self.log = log
        self.log_data = log_data
        self.tcex = tcex
        # TODO: validate this
        self.tcex.logger.update_handler_level('error')

        # properties
        self._redis = None
        self._threatconnect = None
        self.max_diff = 10

    @staticmethod
    def _string_to_int_float(x):
        """Take string input and return float or int."""
        try:
            f = float(x)
            i = int(x)
        except ValueError:
            return x  # return original value
        else:
            if f != i:
                return f  # return float
            return i  # return int

    def details(self, app_data, test_data, op):
        """Return details about the validation."""
        details = ''
        if app_data is not None and test_data is not None and op in ['eq', 'ne']:
            try:
                diff_count = 0
                for i, diff in enumerate(difflib.ndiff(app_data, test_data)):
                    if diff[0] == ' ':  # no difference
                        continue
                    elif diff[0] == '-':
                        details += '\n    * Missing data at index {}'.format(i)
                        self.log_data(
                            'validate',
                            'App Data',
                            '({}), Type: [{}]'.format(app_data, type(app_data)),
                        )
                    elif diff[0] == '+':
                        details += '\n    * Extra data at index {}'.format(i)
                        self.log_data(
                            'validate',
                            'Diff',
                            ('[validate] Diff       : Extra data at index {}'.format(i)),
                        )
                    if diff_count > self.max_diff:
                        details += '\n    * Max number of differences reached.'
                        # don't spam the logs if string are vastly different
                        self.log_data(
                            'validate', 'Maximum Reached', 'Max number of differences reached.'
                        )
                        break
                    diff_count += 1
            except TypeError:
                pass
            except KeyError:
                pass
        return details

    def get_operator(self, op):
        """Get the corresponding operator"""
        operators = {
            'dd': self.operator_deep_diff,
            'eq': self.operator_eq,
            '=': self.operator_eq,
            'le': self.operator_le,
            '<=': self.operator_le,
            'lt': self.operator_lt,
            '<': self.operator_lt,
            'ge': self.operator_ge,
            '>=': self.operator_ge,
            'gt': self.operator_gt,
            '>': self.operator_gt,
            'jeq': self.operator_json_eq,
            'json_eq': self.operator_json_eq,
            'kveq': self.operator_keyvalue_eq,
            'keyvalue_eq': self.operator_keyvalue_eq,
            'ne': self.operator_ne,
            '!=': self.operator_ne,
            'rex': self.operator_regex_match,
        }
        return operators.get(op, None)

    def operator_deep_diff(self, app_data, test_data, **kwargs):
        """Compare app data equals tests data.

        Args:
            app_data (dict|str|list): The data created by the App.
            test_data (dict|str|list): The data provided in the test case.
            exclude_keys (list, kwargs): A list of key for a KeyValueArray to be removed.

        Returns:
            bool: The results of the operator.
        """
        try:
            from deepdiff import DeepDiff
        except ImportError:
            self.log.error('Could not import DeepDiff module (try "pip install deepdiff").')
            return False

        # run operator
        try:
            ddiff = DeepDiff(app_data, test_data, ignore_order=True, **kwargs)
        except KeyError:
            return False, 'Encountered KeyError when running deepdiff'
        except NameError:
            return False, 'Encountered NameError when running deepdiff'

        if ddiff:
            return False, str(ddiff)
        return True, ''

    def operator_eq(self, app_data, tests_data):
        """Compare app data is equal to tests data.

        Args:
            app_data (dict, list, str): The data created by the App.
            test_data (dict, list, str): The data provided in the test case.

        Returns:
            bool, str: The results of the operator and any error message
        """
        results = operator.eq(app_data, tests_data)
        return results, self.details(app_data, tests_data, 'eq')

    def operator_ge(self, app_data, test_data):
        """Compare app data is greater than or equal to tests data.

        Args:
            app_data (dict, list, str): The data created by the App.
            test_data (dict, list, str): The data provided in the test case.

        Returns:
            bool, str: The results of the operator and any error message
        """
        app_data = self._string_to_int_float(app_data)
        test_data = self._string_to_int_float(test_data)
        results = operator.ge(app_data, test_data)
        details = ''
        if not results:
            details = '{} {} !(>=) {} {}'.format(
                app_data, type(app_data), test_data, type(test_data)
            )
        return results, details

    def operator_gt(self, app_data, test_data):
        """Compare app data is greater than tests data.

        Args:
            app_data (dict, list, str): The data created by the App.
            test_data (dict, list, str): The data provided in the test case.

        Returns:
            bool, str: The results of the operator and any error message
        """
        app_data = self._string_to_int_float(app_data)
        test_data = self._string_to_int_float(test_data)
        results = operator.gt(app_data, test_data)
        details = ''
        if not results:
            details = '{} {} !(>) {} {}'.format(
                app_data, type(app_data), test_data, type(test_data)
            )
        return results, details

    def operator_json_eq(self, app_data, test_data, **kwargs):
        """Compare app data equals tests data.

        Takes a str, dict, or list value and removed field before passing to deepdiff. Only fields
        at the "root" level can be removed (e.g., "date", not "data.date").

        Args:
            app_data (dict|str|list): The data created by the App.
            test_data (dict|str|list): The data provided in the test case.

        Returns:
            bool: The results of the operator.
        """
        if isinstance(app_data, (string_types)):
            app_data = json.loads(app_data)
        if isinstance(test_data, (string_types)):
            test_data = json.loads(test_data)

        exclude = kwargs.get('exclude', [])
        if isinstance(app_data, list) and isinstance(test_data, list):
            app_data = [self.operator_json_eq_exclude(ad, exclude) for ad in app_data]
            test_data = [self.operator_json_eq_exclude(td, exclude) for td in test_data]
        elif isinstance(app_data, dict) and isinstance(test_data, dict):
            app_data = self.operator_json_eq_exclude(app_data, exclude)
            test_data = self.operator_json_eq_exclude(test_data, exclude)

        return self.operator_deep_diff(app_data, test_data)

    @staticmethod
    def operator_json_eq_exclude(data, exclude):
        """Remove excluded field from dictionary.

        Args:
            app_data (dict|str): The data to be processed.
            exclude (list): The key names to be "excluded" from data.

        Returns:
            dict: The data with excluded values removed.
        """
        if isinstance(data, (string_types)):
            data = json.loads(data)
        for e in exclude:
            try:
                del data[e]
            except KeyError:
                pass
        return data

    def operator_keyvalue_eq(self, app_data, test_data, **kwargs):
        """Compare app data equals tests data.

        Args:
            app_data (dict|str|list): The data created by the App.
            test_data (dict|str|list): The data provided in the test case.

        Returns:
            bool: The results of the operator.
        """
        # remove exclude_key field. usually dynamic data like date fields.
        if kwargs.get('exclude_keys') is not None:
            app_data = [
                kv for kv in app_data if kv.get('key') not in kwargs.get('exclude_keys', [])
            ]
            test_data = [
                kv for kv in test_data if kv.get('key') not in kwargs.get('exclude_keys', [])
            ]
            del kwargs['exclude_keys']

        return self.operator_deep_diff(app_data, test_data, **kwargs)

    def operator_le(self, app_data, test_data):
        """Compare app data is less than or equal to tests data.

        Args:
            app_data (dict, list, str): The data created by the App.
            test_data (dict, list, str): The data provided in the test case.

        Returns:
            bool, str: The results of the operator and any error message
        """
        app_data = self._string_to_int_float(app_data)
        test_data = self._string_to_int_float(test_data)
        results = operator.le(app_data, test_data)
        details = ''
        if not results:
            details = '{} {} !(<=) {} {}'.format(
                app_data, type(app_data), test_data, type(test_data)
            )
        return results, details

    def operator_lt(self, app_data, test_data):
        """Compare app data is less than tests data.

        Args:
            app_data (dict, list, str): The data created by the App.
            test_data (dict, list, str): The data provided in the test case.

        Returns:
            bool, str: The results of the operator and any error message
        """
        app_data = self._string_to_int_float(app_data)
        test_data = self._string_to_int_float(test_data)
        results = operator.lt(app_data, test_data)
        details = ''
        if not results:
            details = '{} {} !(<) {} {}'.format(
                app_data, type(app_data), test_data, type(test_data)
            )
        return results, details

    def operator_ne(self, app_data, tests_data):
        """Compare app data is not equal to tests data.

        Args:
            app_data (dict, list, str): The data created by the App.
            test_data (dict, list, str): The data provided in the test case.

        Returns:
            bool, str: The results of the operator and any error message
        """
        results = operator.ne(app_data, tests_data)
        return results, self.details(app_data, tests_data, 'eq')

    @staticmethod
    def operator_regex_match(app_data, test_data):
        """Compare app data equals tests data.

        Args:
            app_data (dict|str|list): The data created by the App.
            test_data (dict|str|list): The data provided in the test case.

        Returns:
            bool: The results of the operator.
        """
        if re.match(test_data, app_data) is None:
            return False, 'app_data id not match regex ({})'.format(test_data)
        return True, ''

    @property
    def redis(self):
        """Get the current instance of Redis for validating data"""
        if not self._redis:
            self._redis = Redis(self)
        return self._redis

    @property
    def threatconnect(self):
        """Get the current instance of ThreatConnect for validating data"""
        if not self._threatconnect:
            self._threatconnect = ThreatConnect(self)
        return self._threatconnect


class Redis(object):
    """Validates Redis data"""

    def __init__(self, provider, truncate=50):
        """Initialize class properties."""
        self.provider = provider
        self.truncate = truncate

        # Properties
        self.log_data = self.provider.log_data
        self.redis_client = provider.tcex.playbook.db.r

    def not_null(self, variable):
        """Validate that a variable is not empty/null"""
        # Could do something like self.ne(variable, None), but want to be pretty specific on
        # the errors on this one
        variable_data = self.provider.tcex.playbook.read(variable)
        self.log_data('validate', 'Variable', variable)
        self.log_data('validate', 'DB Data', variable_data)
        if not variable:
            self.provider.log.error('NoneError: Redis Variable not provided')
            return False

        if not variable_data:
            self.provider.log.error(
                'NotFoundError: Redis Variable {} was not found.'.format(variable)
            )
            return False

        return True

    def type(self, variable):
        """Validate the type of a redis variable"""
        variable_data = self.provider.tcex.playbook.read(variable)
        self.log_data('validate', 'Variable', variable)
        self.log_data('validate', 'DB Data', variable_data)
        redis_type = self.provider.tcex.playbook.variable_type(variable)
        if redis_type.endswith('Array'):
            redis_type = list
        elif redis_type.startswith('String'):
            redis_type = str
        elif redis_type.startswith('KeyValuePair'):
            redis_type = dict
        else:
            redis_type = str

        if not variable_data:
            self.provider.log.error(
                'NotFoundError: Redis Variable {} was not found.'.format(variable)
            )
            return False
        if not isinstance(variable_data, redis_type):
            self.provider.log.error(
                'TypeMismatchError: Redis Type: {} and Variable: {} '
                'do not match'.format(redis_type, variable)
            )
            return False

        return True

    def data(self, variable, test_data, op=None, **kwargs):
        """Validate Redis data <operator> test_data.

        Args:
            variable (str): The variable to read from REDIS.
            data (dict or list or str): The validation data
            op (str, optional): The comparison operator expression. Defaults to "eq".

        Returns:
            [type]: [description]
        """
        op = op or 'eq'
        if not variable:
            self.provider.log.error('NoneError: Redis Variable not provided')
            return False

        if not self.provider.get_operator(op):
            self.provider.log.error('Invalid operator provided ({})'.format(op))
            return False

        if variable.endswith('Binary'):
            app_data = self.provider.tcex.playbook.read_binary(variable, False, False)
        elif variable.endswith('BinaryArray'):
            app_data = self.provider.tcex.playbook.read_binary_array(variable, False, False)
        else:
            app_data = self.provider.tcex.playbook.read(variable)

        # logging header
        self.provider.log.info('{0} {1} {0}'.format('-' * 10, variable))

        # run operator
        passed, details = self.provider.get_operator(op)(app_data, test_data, **kwargs)

        # log validation data in a readable format
        self.validate_log_output(passed, app_data, test_data, details.strip(), op)

        # build assert error
        assert_error = '\n App Data     : {}\n Expected Data: {}\n Details      : {}\n'.format(
            app_data, test_data, details
        )
        return passed, assert_error

    def eq(self, variable, data):
        """Validate test data equality"""
        return self.data(variable, data)

    def dd(self, variable, data, **kwargs):
        """Validate test data equality"""
        return self.data(variable, data, op='dd', **kwargs)

    def ge(self, variable, data):
        """Validate test data equality"""
        return self.data(variable, data, op='ge')

    def gt(self, variable, data):
        """Validate test data equality"""
        return self.data(variable, data, op='gt')

    def jeq(self, variable, data, **kwargs):
        """Validate JSON data equality"""
        return self.data(variable, data, op='jeq', **kwargs)

    def json_eq(self, variable, data, **kwargs):
        """Validate JSON data equality"""
        return self.data(variable, data, op='jeq', **kwargs)

    def kveq(self, variable, data, **kwargs):
        """Validate JSON data equality"""
        return self.data(variable, data, op='kveq', **kwargs)

    def keyvalue_eq(self, variable, data, **kwargs):
        """Validate JSON data equality"""
        return self.data(variable, data, op='kveq', **kwargs)

    def lt(self, variable, data):
        """Validate test data less than"""
        return self.data(variable, data, op='lt')

    def le(self, variable, data):
        """Validate test data less than or equal"""
        return self.data(variable, data, op='le')

    def ne(self, variable, data):
        """Validate test data non equality"""
        return self.data(variable, data, op='ne')

    def rex(self, variable, data):
        """Test App data with regex"""
        return self.data(variable, r'{}'.format(data), op='rex')

    def validate_log_output(self, passed, app_data, test_data, details, op):
        """Format the validation log output to be easier to read.

        Args:
            passed (bool): The results of the validation test.
            app_data (str): The data store in Redis.
            test_data (str): The user provided data.
            op (str): The comparison operator.

        Raises:
            RuntimeError: Raise error on validation failure if halt_on_fail is True.
        """
        truncate = self.truncate
        if app_data is not None and passed:
            if isinstance(app_data, (string_types)) and len(app_data) > truncate:
                app_data = app_data[:truncate]
            elif isinstance(app_data, (list)):
                db_data_truncated = []
                for d in app_data:
                    if d is not None and isinstance(d, string_types) and len(d) > truncate:
                        db_data_truncated.append('{} ...'.format(d[: self.truncate]))
                    else:
                        db_data_truncated.append(d)
                app_data = db_data_truncated

        if test_data is not None and passed:
            if isinstance(test_data, (string_types)) and len(test_data) > truncate:
                test_data = test_data[: self.truncate]
            elif isinstance(test_data, (list)):
                user_data_truncated = []
                for u in test_data:
                    if isinstance(app_data, (string_types)) and len(u) > truncate:
                        user_data_truncated.append('{} ...'.format(u[: self.truncate]))
                    else:
                        user_data_truncated.append(u)
                test_data = user_data_truncated

        self.log_data('validate', 'App Data', '({}), Type: [{}]'.format(app_data, type(app_data)))
        self.log_data('validate', 'Operator', op)
        self.log_data('validate', 'Test Data', '({}), Type: [{}]'.format(test_data, type(app_data)))

        if passed:
            self.log_data('validate', 'Result', 'Passed')
        else:
            self.log_data('validate', 'Result', 'Failed', 'error')
            self.log_data('validate', 'Details', details, 'error')


class ThreatConnect(object):
    """Validate ThreatConnect data"""

    def __init__(self, provider):
        """Initialize class properties"""
        self.provider = provider

    def batch(self, context, owner, validation_criteria):
        """Validates the batch submission"""

        validation_percent = validation_criteria.get('percent', 100)
        validation_count = validation_criteria.get('count', None)
        batch_submit_totals = self._get_batch_submit_totals(context)

        if validation_count:
            validation_percent = self._convert_to_percent(validation_count, batch_submit_totals)

        batch_errors = []
        for filename in os.listdir(os.path.join('.', 'log', context)):
            with open(os.path.join('.', 'log', context, filename), 'r') as fh:
                if not filename.startswith('errors-') or not filename.endswith('.json'):
                    continue
                batch_errors += json.load(fh)

        for filename in os.listdir(os.path.join('.', 'log', context)):
            if not filename.startswith('batch-') or not filename.endswith('.json'):
                continue

            with open(os.path.join('.', 'log', context, filename), 'r') as fh:
                data = json.load(fh)
                validation_data = self._partition_batch_data(data)
                sample_validation_data = []
                for key in validation_data:
                    for sub_partition in validation_data.get(key).values():
                        sample_size = math.ceil(len(sub_partition) * (validation_percent / 100))
                        sample_validation_data.extend(random.sample(sub_partition, sample_size))

                files = []
                for sample_data in sample_validation_data:
                    sample_data_type = sample_data.get('type').lower()
                    if sample_data_type not in ['document', 'report']:
                        files.append(None)
                        continue

                    if sample_data_type == 'document':
                        sample_data_type = 'documents'
                    else:
                        sample_data_type = 'reports'

                    filename = (
                        sample_data_type
                        + '--'
                        + sample_data.get('xid')
                        + '--'
                        + sample_data.get('name', '')
                    )
                    filename = os.path.join('.', 'log', context, filename)
                    if os.path.isfile(filename):
                        files.append(filename)
                    else:
                        files.append(None)
                results = self.tc_entities(sample_validation_data, owner, files=files)
                for result in results:
                    if result.get('valid'):
                        continue
                    name = result.get('name')
                    batch_error = self._batch_error(name, batch_errors)
                    if batch_error:
                        self.provider.log.error(
                            'Errors validating {} due to batch '
                            'submission error: {}'.format(name, batch_error)
                        )
                        continue
                    # TODO: Should use pip install pytest-check is_true method so it wont fail after
                    # one failed assert.
                    assert result.get(
                        'valid'
                    ), '{} in ThreatConnect did not match what was submitted. Errors:{}'.format(
                        name, result.get('errors')
                    )
        self._log_batch_submit_details(batch_errors, owner)

    def _log_batch_submit_details(self, batch_errors, owner):
        if not batch_errors:
            return

        counts = {}
        error_regex = r'\((.*?)\)'
        for error in batch_errors:
            reason = error.get('errorReason', '')
            m = re.search(error_regex, reason)
            if not m:
                continue
            if not m.group(1) in counts:
                counts[m.group(1)] = 0
            counts[m.group(1)] += 1
        log_message = ''
        for code, count in counts.items():
            log_message += (
                self.provider.tcex.batch(owner).error_codes.get(code) + ': ' + str(count) + '\n'
            )

        self.provider.log.error(log_message)

    @staticmethod
    def _batch_error(key, batch_errors):
        for error in batch_errors:
            reason = error.get('errorReason', '')
            if key in reason:
                return reason
        return None

    def dir(self, directory, owner):
        """Validate the content of a given dir"""
        results = []
        for test_file in os.listdir(directory):
            if not (test_file.endswith('.json') and test_file.startswith('validate_')):
                continue
            results.append(self.file('{}/{}'.format(directory, test_file), owner))
        return results

    def file(self, file, owner):
        """Validate the content of a given file"""
        entities = self._convert_to_entities(file)
        return self.tc_entities(entities, owner)

    def tc_entities(self, tc_entities, owner, files=None):
        """Validate a array of tc_entities"""
        results = []
        if files:
            if not len(tc_entities) == len(files):
                return [
                    {
                        'valid': False,
                        'errors': [
                            'LengthError: Length of files provided does not '
                            'match length of entities provided.'
                        ],
                    }
                ]

        for index, entity in enumerate(tc_entities):
            name = entity.get('name', entity.get('summary'))
            if files:
                valid, errors = self.tc_entity(entity, owner, files[index])
            else:
                valid, errors = self.tc_entity(entity, owner)
            results.append({'name': name, 'valid': valid, 'errors': errors})
        return results

    def tc_entity(self, tc_entity, owner, file=None):
        """Validate the ti_response entity"""
        parameters = {'includes': ['additional', 'attributes', 'labels', 'tags']}
        ti_entity = self._convert_to_ti_entity(tc_entity, owner)
        ti_response = ti_entity.single(params=parameters)
        entity_name = tc_entity.get('name')
        errors = []
        if not self.success(ti_response):
            error = 'NotFoundError: Provided {}: {} could not be fetched from ThreatConnect'.format(
                tc_entity.get('type'), tc_entity.get('summary', entity_name)
            )
            return False, [error]

        ti_response_entity = None
        ti_response = ti_response.json().get('data', {}).get(ti_entity.api_entity, {})
        for entity in self.provider.tcex.ti.entities(ti_response, tc_entity.get('type', None)):
            ti_response_entity = entity
            # pylint: disable=W0612
            valid_attributes, attributes_errors = self._response_attributes(ti_response, tc_entity)
            # pylint: disable=W0612
            valid_tags, tag_errors = self._response_tags(ti_response, tc_entity)
            # pylint: disable=W0612
            valid_labels, label_errors = self._response_labels(ti_response, tc_entity)
            # pylint: disable=W0612
            valid_file, file_errors = self._file(ti_entity, file)

            errors = attributes_errors + tag_errors + label_errors + file_errors

        if ti_entity.type == 'Indicator':
            provided_rating = tc_entity.get('rating', None)
            expected_rating = ti_response.get('rating', None)
            if not provided_rating == expected_rating:
                error = 'RatingError: Provided rating {} does not match ' 'actual rating {}'.format(
                    provided_rating, expected_rating
                )
                errors += error

            provided_confidence = tc_entity.get('confidence', None)
            expected_confidence = ti_response.get('confidence', None)
            if not provided_confidence == expected_confidence:
                error = (
                    'ConfidenceError: Provided confidence {} does not match '
                    'actual confidence {}'.format(provided_confidence, expected_confidence)
                )
                errors += error

            provided_summary = unquote(
                ':'.join([x for x in ti_entity.unique_id.split(':') if x.strip()])
            )
            expected_summary = unquote(ti_response_entity.get('value', ''))
            if provided_summary != expected_summary:
                error = (
                    'SummaryError: Provided summary {} does not match '
                    'actual summary {}'.format(provided_summary, expected_summary)
                )
                errors += error
        elif ti_entity.type == 'Group':
            provided_summary = tc_entity.get('name', None)
            expected_summary = ti_response_entity.get('value', None)
            if not provided_summary == expected_summary:
                error = (
                    'SummaryError: Provided summary {} does not match '
                    'actual summary {}'.format(provided_summary, expected_summary)
                )
                errors += error

        return not bool(errors), errors

    def flatten(self, lis):
        """Idk why python doesnt have this built in but helper function to flatten a list"""
        new_lis = []
        for item in lis:
            if isinstance(item, list):
                new_lis.extend(self.flatten(item))
            else:
                new_lis.append(item)
        return new_lis

    @staticmethod
    def compare_dicts(expected, actual, error_type=''):
        """Compare two dicts and returns a list of errors if they don't match"""
        errors = []
        for item in expected:
            if item in actual:
                if str(expected.get(item)) != actual.get(item):
                    errors.append(
                        '{0}{1} : {2} did not match {1} : {3}'.format(
                            error_type, item, expected.get(item), actual.get(item)
                        )
                    )
                actual.pop(item)
            else:
                errors.append(
                    '{}{} : {} was in expected results but not in actual results.'.format(
                        error_type, item, expected.get(item)
                    )
                )
        for item in list(actual.items()):
            errors.append(
                '{}{} : {} was in actual results but not in expected results.'.format(
                    error_type, item, actual.get(item)
                )
            )

        return bool(errors), errors

    @staticmethod
    def compare_lists(expected, actual, error_type=''):
        """Compare two lists and returns a list of errors if they don't match"""
        errors = []
        for item in expected:
            if item in actual:
                actual.remove(item)
            else:
                errors.append(
                    '{}{} was in expected results but not in actual results.'.format(
                        error_type, item
                    )
                )
        for item in actual:
            errors.append(
                '{}{} was in actual results but not in expected results.'.format(error_type, item)
            )

        return bool(errors), errors

    @staticmethod
    def _convert_to_entities(file):
        """Convert file to tc_entity array"""
        with open(file, 'r') as read_file:
            data = json.load(read_file)
        return data

    def _custom_indicators(self):
        indicator_details = self.provider.tcex.indicator_types_data
        custom_indicators = []
        for indicator in indicator_details.values():
            if indicator.get('custom', False):
                custom_indicators.append(indicator.get('name'))

        return custom_indicators

    def _convert_to_ti_entity(self, tc_entity, owner):
        """Convert a tc_entity to a ti_entity"""
        ti_entity = None

        if tc_entity.get('type') in self.provider.tcex.group_types:
            # We can't search by xid sadly so have to search by name and validate xid to
            # get the id of the group.
            filters = self.provider.tcex.ti.filters()
            filters.add_filter('name', '=', tc_entity.get('name'))
            generic_group_entity = self.provider.tcex.ti.group(
                group_type=tc_entity.get('type'), owner=owner
            )
            parameters = {'includes': ['additional']}
            for entity in generic_group_entity.many(filters=filters, params=parameters):
                if entity.get('xid') == tc_entity.get('xid'):
                    ti_entity = self.provider.tcex.ti.group(
                        group_type=tc_entity.get('type'),
                        owner=owner,
                        name=entity.get('name'),
                        unique_id=entity.get('id'),
                    )
        elif tc_entity.get('type') in self.provider.tcex.indicator_types:
            tc_entity['summary'] = tc_entity.get('summary')
            if tc_entity.get('type').lower() == 'file':
                tc_entity['summary'] = tc_entity.get('summary').upper()
            ti_entity = self.provider.tcex.ti.indicator(
                indicator_type=tc_entity.get('type'),
                owner=owner,
                unique_id=tc_entity.get('summary'),
            )
        elif tc_entity.get('type') == 'Victim':
            # TODO: Will need to do something similar to what was done to get the groups entity.
            pass

        return ti_entity

    def _get_batch_submit_totals(self, context):
        """Breaks the batch submitions up into seperate partitions with each partition containing
        its total."""
        counts = {}
        for filename in os.listdir(os.path.join('.', 'log', context)):
            if not filename.startswith('batch-') or not filename.endswith('.json'):
                continue
            with open(os.path.join('.', 'log', context, filename), 'r') as fh:
                data = json.load(fh)
                partitioned_data = self._partition_batch_data(data)
                for key in partitioned_data:
                    if key not in counts:
                        counts[key] = {}
                    for sub_key in partitioned_data.get(key):
                        if sub_key not in counts:
                            counts[key][sub_key] = 0
                        counts[key][sub_key] += len(partitioned_data[key][sub_key])
        return counts

    @staticmethod
    def _convert_to_percent(validation_count, batch_submit_totals):
        """Given a count, calculate what percentage of the batch submits that is"""
        total = 0
        for key in batch_submit_totals.keys():
            for count in batch_submit_totals.get(key).values():
                total += count
        if validation_count > total:
            return 100
        return round((validation_count / total) * 100, 2)

    @staticmethod
    def _partition_batch_data(data):
        """Partitions the batch submittions into batch_submit[type][subtype] partitions"""
        partitioned_data = {}
        for key in data.keys():
            partitioned_sub_data = {}
            for sub_data in data.get(key):
                if not sub_data.get('type') in partitioned_sub_data.keys():
                    partitioned_sub_data[sub_data.get('type')] = [sub_data]
                else:
                    partitioned_sub_data[sub_data.get('type')].append(sub_data)
            partitioned_data[key] = partitioned_sub_data
        return partitioned_data

    def _response_attributes(self, ti_response, tc_entity):
        """Validate the ti_response attributes"""
        if not ti_response or not tc_entity:
            return True

        expected = {}
        actual = {}
        for attribute in tc_entity.get('attribute', []):
            expected[attribute.get('type')] = attribute.get('value')
        for attribute in ti_response.get('attribute', []):
            actual[attribute.get('type')] = attribute.get('value')
        return self.compare_dicts(expected, actual, error_type='AttributeError: ')

    def _response_tags(self, ti_response, tc_entity):
        """Validate the ti_response tags"""
        if not ti_response or not tc_entity:
            return True

        expected = []
        actual = []
        for tag in tc_entity.get('tag', []):
            expected.append(tag.get('name'))
        for tag in ti_response.get('tag', []):
            actual.append(tag.get('name'))

        return self.compare_lists(expected, actual, error_type='TagError: ')

    def _response_labels(self, ti_response, tc_entity):
        """Validate the ti_response labels"""
        if not ti_response or not tc_entity:
            return True

        expected = []
        actual = []
        for tag in tc_entity.get('securityLabel', []):
            expected.append(tag)
        for tag in ti_response.get('securityLabel', []):
            actual.append(tag.get('name'))

        return self.compare_lists(expected, actual, error_type='SecurityLabelError: ')

    @staticmethod
    def _file(ti_entity, file):
        """Handle file data"""
        if not file:
            return True, []

        errors = []
        if ti_entity.api_sub_type == 'Document' or ti_entity.api_sub_type == 'Report':
            actual_hash = ti_entity.get_file_hash()
            actual_hash = actual_hash.hexdigest()
            provided_hash = hashlib.sha256()
            with open(file, 'rb') as f:
                for byte_block in iter(lambda: f.read(4096), b''):
                    provided_hash.update(byte_block)
            provided_hash = provided_hash.hexdigest()
            if not provided_hash == actual_hash:
                errors.append(
                    'sha256 {} of provided file did not match sha256 of '
                    'actual file {}'.format(provided_hash, actual_hash)
                )
        return bool(errors), errors

    @staticmethod
    def success(r):
        """

        Args:
            r:

        Return:

        """
        status = True
        if r.ok:
            try:
                if r.json().get('status') != 'Success':
                    status = False
            except Exception:
                status = False
        else:
            status = False
        return status



# -*- coding: utf-8 -*-
"""Test the TcEx Threat Intel Module."""

from ..tcex_init import tcex


# pylint: disable=W0201
class TestAddressIndicators:
    """Test TcEx Address Indicators."""

    def setup_class(self):
        """Configure setup before all tests."""
        self.ti = tcex.ti
        addresses = [
            '11.21.31.41',
            '12.11.12.11',
            '12.13.14.15',
            '13.11.13.11',
            '13.23.33.43',
            '14.111.14.15',
            '17.15.30.41',
            '40.30.20.10',
        ]
        for ip in addresses:
            ti = self.ti.indicator(indicator_type='Address', owner=tcex.args.tc_owner, ip=ip)
            ti.delete()

    def test_address_get(self, ip='17.15.30.41'):
        """Test address get."""
        # create
        self.address_create(ip)

        # get
        ti = self.ti.address(ip, owner=tcex.args.tc_owner)
        r = ti.single()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('address').get('ip') == ip

    def test_address_get_attributes(self, ip='11.21.31.41'):
        """Test address get."""
        # create
        self.address_create(ip)
        self.test_address_add_attribute(False, ip, 'Description', 'test1')
        self.test_address_add_attribute(False, ip, 'Description', 'test2')
        self.test_address_add_attribute(False, ip, 'Description', 'test3')

        # get attributes
        ti = self.ti.address(ip, owner=tcex.args.tc_owner)
        for attribute in ti.attributes():
            assert attribute
            break
        else:
            assert False

    def test_address_get_tags(self, ip='13.23.33.43'):
        """Test address get."""
        # create
        self.address_create(ip)
        self.test_address_add_tag(False, ip, 'One')
        self.test_address_add_tag(False, ip, 'Two')

        # get tags
        ti = self.ti.address(ip, owner=tcex.args.tc_owner)
        for tag in ti.tags():
            assert tag.get('name')
            break
        else:
            assert False

    def test_address_get_include(self, ip='40.30.20.10'):
        """Test address get."""
        self.address_create(ip)

        self.test_address_add_attribute(False, ip, 'Description', 'test123')
        self.test_address_add_label(False, ip, 'TLP:RED')
        self.test_address_add_tag(False, ip, 'PyTest')

        parameters = {'includes': ['additional', 'attributes', 'labels', 'tags']}
        ti = self.ti.address(ip, owner=tcex.args.tc_owner)
        r = ti.single(params=parameters)
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('address').get('ip') == ip
        assert ti_data.get('data').get('address').get('attribute')[0].get('value') == 'test123'
        assert ti_data.get('data').get('address').get('securityLabel')[0].get('name') == 'TLP:RED'
        assert ti_data.get('data').get('address').get('tag')[0].get('name') == 'PyTest'

    def address_create(self, ip='14.111.14.15'):
        """Test address create."""
        ti = self.ti.indicator(indicator_type='Address', owner=tcex.args.tc_owner, ip=ip)
        r = ti.create()
        ti_data = r.json()
        assert r.status_code == 201
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('address').get('ip') == ip

    def test_address_add_attribute(
        self,
        should_create=True,
        ip='12.13.14.15',
        attribute_type='Description',
        attribute_value='Example Description.',
    ):
        """Test address attribute add."""
        if should_create:
            self.address_create(ip)

        ti = self.ti.address(ip, owner=tcex.args.tc_owner)
        r = ti.add_attribute(attribute_type=attribute_type, attribute_value=attribute_value)
        attribute_data = r.json()
        assert r.status_code == 201
        assert attribute_data.get('status') == 'Success'
        assert attribute_data.get('data').get('attribute').get('value') == attribute_value

    def test_address_add_label(self, should_create=True, ip='12.13.14.15', label='TLP:GREEN'):
        """Test address attribute add."""
        if should_create:
            self.address_create(ip)

        ti = self.ti.address(ip, owner=tcex.args.tc_owner)
        r = ti.add_label(label=label)
        label_data = r.json()
        assert r.status_code == 201
        assert label_data.get('status') == 'Success'

    def test_address_add_tag(self, should_create=True, ip='12.13.14.15', name='Crimeware'):
        """Test address attribute add."""
        if should_create:
            self.address_create(ip)

        ti = self.ti.address(ip, owner=tcex.args.tc_owner)
        r = ti.add_tag(name=name)
        tag_data = r.json()
        assert r.status_code == 201
        assert tag_data.get('status') == 'Success'

    def address_delete(self, ip='12.11.12.11'):
        """Test address delete."""
        # create indicator
        self.address_create(ip)

        # delete indicator
        ti = self.ti.indicator(indicator_type='Address', owner=tcex.args.tc_owner, ip=ip)
        r = ti.delete()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'

    def test_address_update(self, ip='13.11.13.11'):
        """Test address update."""
        # create indicator
        self.address_create(ip)

        # update indicator
        ti = self.ti.indicator(
            indicator_type='Address', owner=tcex.args.tc_owner, ip=ip, rating=5, confidence=10
        )
        r = ti.update()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('address').get('rating') == 5.0
        assert ti_data.get('data').get('address').get('confidence') == 10
